@@ -34,12 +34,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdlib.h>
 #include <signal.h>
 
-#ifdef KWIN_BUILD_SCRIPTING
-#include "scripting/client.h"
-#include "scripting/scripting.h"
-#include "scripting/workspaceproxy.h"
-#endif
-
 #include "bridge.h"
 #include "group.h"
 #include "workspace.h"
@@ -101,6 +95,7 @@ Client::Client(Workspace* ws)
     , transient_for (NULL)
     , transient_for_id(None)
     , original_transient_for_id(None)
+    , shade_below(NULL)
     , skip_switcher(false)
     , blocks_compositing(false)
     , autoRaiseTimer(NULL)
@@ -108,10 +103,10 @@ Client::Client(Workspace* ws)
     , delayedMoveResizeTimer(NULL)
     , in_group(NULL)
     , window_group(None)
-    , client_group(NULL)
+    , tab_group(NULL)
     , in_layer(UnknownLayer)
     , ping_timer(NULL)
-    , process_killer(NULL)
+    , m_killHelperPID(0)
     , user_time(CurrentTime)   // Not known yet
     , allowed_actions(0)
     , block_geometry_updates(0)
@@ -129,16 +124,13 @@ Client::Client(Workspace* ws)
     , demandAttentionKNotifyTimer(NULL)
     , m_responsibleForDecoPixmap(false)
     , paintRedirector(0)
+    , m_firstInTabBox(false)
     , electricMaximizing(false)
     , activitiesDefined(false)
     , needsSessionInteract(false)
     , input_window(None)
 {
     // TODO: Do all as initialization
-
-#ifdef KWIN_BUILD_SCRIPTING
-    scriptCache = new QHash<QScriptEngine*, ClientResolution>();
-#endif
 #ifdef HAVE_XSYNC
     syncRequest.counter = syncRequest.alarm = None;
     syncRequest.timeout = syncRequest.failsafeTimeout = NULL;
@@ -148,7 +140,6 @@ Client::Client(Workspace* ws)
     // Set the initial mapping state
     mapping_state = Withdrawn;
     quick_tile_mode = QuickTileNone;
-    geom_pretile = QRect(0, 0, 0, 0);
     desk = 0; // No desktop yet
 
     mode = PositionCenter;
@@ -188,7 +179,6 @@ Client::Client(Workspace* ws)
     skip_pager = false;
 
     max_mode = MaximizeRestore;
-    maxmode_restore = MaximizeRestore;
 
     cmap = None;
 
@@ -197,8 +187,7 @@ Client::Client(Workspace* ws)
 
 #ifdef KWIN_BUILD_TABBOX
     // TabBoxClient
-    m_tabBoxClient = QSharedPointer<TabBox::TabBoxClientImpl>(new TabBox::TabBoxClientImpl());
-    m_tabBoxClient->setClient(this);
+    m_tabBoxClient = QSharedPointer<TabBox::TabBoxClientImpl>(new TabBox::TabBoxClientImpl(this));
 #endif
 
     geom = QRect(0, 0, 100, 100);   // So that decorations don't start with size being (0,0)
@@ -208,6 +197,8 @@ Client::Client(Workspace* ws)
     connect(this, SIGNAL(geometryShapeChanged(KWin::Toplevel*,QRect)), SIGNAL(geometryChanged()));
     connect(this, SIGNAL(clientMaximizedStateChanged(KWin::Client*,KDecorationDefines::MaximizeMode)), SIGNAL(geometryChanged()));
     connect(this, SIGNAL(clientStepUserMovedResized(KWin::Client*,QRect)), SIGNAL(geometryChanged()));
+    connect(this, SIGNAL(clientStartUserMovedResized(KWin::Client*)), SIGNAL(moveResizedChanged()));
+    connect(this, SIGNAL(clientFinishUserMovedResized(KWin::Client*)), SIGNAL(moveResizedChanged()));
 
     // SELI TODO: Initialize xsizehints??
 }
@@ -230,9 +221,6 @@ Client::~Client()
     assert(block_geometry_updates == 0);
     assert(!check_active_modal);
     delete bridge;
-#ifdef KWIN_BUILD_SCRIPTING
-    delete scriptCache;
-#endif
 }
 
 // Use destroyClient() or releaseWindow(), Client instances cannot be deleted directly
@@ -248,7 +236,10 @@ void Client::releaseWindow(bool on_shutdown)
 {
     assert(!deleting);
     deleting = true;
-    Deleted* del = Deleted::create(this);
+    Deleted* del = NULL;
+    if (!on_shutdown) {
+        del = Deleted::create(this);
+    }
     if (moveResizeMode)
         emit clientFinishUserMovedResized(this);
     emit windowClosed(this, del);
@@ -272,15 +263,14 @@ void Client::releaseWindow(bool on_shutdown)
     XUnmapWindow(display(), frameId());  // Destroying decoration would cause ugly visual effect
     destroyDecoration();
     cleanGrouping();
-    if (clientGroup())
-        clientGroup()->remove(this, QRect(), true);
     if (!on_shutdown) {
         workspace()->removeClient(this, Allowed);
         // Only when the window is being unmapped, not when closing down KWin (NETWM sections 5.5,5.7)
         info->setDesktop(0);
         desk = 0;
         info->setState(0, info->state());  // Reset all state flags
-    }
+    } else
+        untab();
     XDeleteProperty(display(), client, atoms->kde_net_wm_user_creation_time);
     XDeleteProperty(display(), client, atoms->net_frame_extents);
     XDeleteProperty(display(), client, atoms->kde_net_wm_frame_strut);
@@ -300,8 +290,10 @@ void Client::releaseWindow(bool on_shutdown)
     XDestroyWindow(display(), frameId());
     //frame = None;
     --block_geometry_updates; // Don't use GeometryUpdatesBlocker, it would now set the geometry
-    disownDataPassedToDeleted();
-    del->unrefWindow();
+    if (!on_shutdown) {
+        disownDataPassedToDeleted();
+        del->unrefWindow();
+    }
     checkNonExistentClients();
     deleteClient(this, Allowed);
     ungrabXServer();
@@ -333,8 +325,6 @@ void Client::destroyClient()
     workspace()->clientHidden(this);
     destroyDecoration();
     cleanGrouping();
-    if (clientGroup())
-        clientGroup()->remove(this, QRect(), true);
     workspace()->removeClient(this, Allowed);
     client = None; // invalidate
     XDestroyWindow(display(), wrapper);
@@ -798,7 +788,7 @@ bool Client::noBorder() const
 
 bool Client::userCanSetNoBorder() const
 {
-    return !isFullScreen() && !isShade() && (clientGroup() == NULL || !(clientGroup()->items().count() > 1));
+    return !isFullScreen() && !isShade() && !tabGroup();
 }
 
 void Client::setNoBorder(bool set)
@@ -810,7 +800,7 @@ void Client::setNoBorder(bool set)
         return;
     noborder = set;
     updateDecoration(true, false);
-    updateWindowRules();
+    updateWindowRules(Rules::NoBorder);
 }
 
 void Client::checkNoBorder()
@@ -976,6 +966,11 @@ bool Client::isMinimizable() const
     return true;
 }
 
+void Client::setMinimized(bool set)
+{
+    set ? minimize() : unminimize();
+}
+
 /**
  * Minimizes this client plus its transients
  */
@@ -987,18 +982,6 @@ void Client::minimize(bool avoid_animation)
     if (isShade()) // NETWM restriction - KWindowInfo::isMinimized() == Hidden && !Shaded
         info->setState(0, NET::Shaded);
 
-#ifdef KWIN_BUILD_SCRIPTING
-    //Scripting call. Does not use a signal/slot mechanism
-    //as ensuring connections was a bit difficult between
-    //so many clients and the workspace
-    SWrapper::WorkspaceProxy* ws_wrap = SWrapper::WorkspaceProxy::instance();
-    if (ws_wrap != 0) {
-        ws_wrap->sl_clientMinimized(this);
-    }
-#endif
-
-    emit s_minimized();
-
     Notify::raise(Notify::Minimize);
 
     minimized = true;
@@ -1006,14 +989,15 @@ void Client::minimize(bool avoid_animation)
     updateVisibility();
     updateAllowedActions();
     workspace()->updateMinimizedOfTransients(this);
-    updateWindowRules();
+    updateWindowRules(Rules::Minimize);
     workspace()->updateFocusChains(this, Workspace::FocusChainMakeLast);
     // TODO: merge signal with s_minimized
     emit clientMinimized(this, !avoid_animation);
 
     // Update states of all other windows in this group
-    if (clientGroup())
-        clientGroup()->updateStates(this);
+    if (tabGroup())
+        tabGroup()->updateStates(this, TabGroup::Minimized);
+    emit minimizedChanged();
 }
 
 void Client::unminimize(bool avoid_animation)
@@ -1028,26 +1012,18 @@ void Client::unminimize(bool avoid_animation)
     if (isShade()) // NETWM restriction - KWindowInfo::isMinimized() == Hidden && !Shaded
         info->setState(NET::Shaded, NET::Shaded);
 
-#ifdef KWIN_BUILD_SCRIPTING
-    SWrapper::WorkspaceProxy* ws_wrap = SWrapper::WorkspaceProxy::instance();
-    if (ws_wrap != 0) {
-        ws_wrap->sl_clientUnminimized(this);
-    }
-#endif
-
-    emit s_unminimized();
-
     Notify::raise(Notify::UnMinimize);
     minimized = false;
     updateVisibility();
     updateAllowedActions();
     workspace()->updateMinimizedOfTransients(this);
-    updateWindowRules();
+    updateWindowRules(Rules::Minimize);
     emit clientUnminimized(this, !avoid_animation);
 
     // Update states of all other windows in this group
-    if (clientGroup())
-        clientGroup()->updateStates(this);
+    if (tabGroup())
+        tabGroup()->updateStates(this, TabGroup::Minimized);
+    emit minimizedChanged();
 }
 
 QRect Client::iconGeometry() const
@@ -1073,8 +1049,14 @@ bool Client::isShadeable() const
     return !isSpecialWindow() && !noBorder() && (rules()->checkShade(ShadeNormal) != rules()->checkShade(ShadeNone));
 }
 
+void Client::setShade(bool set) {
+    set ? setShade(ShadeNormal) : setShade(ShadeNone);
+}
+
 void Client::setShade(ShadeMode mode)
 {
+    if (mode == ShadeHover && isMove())
+        return; // causes geometry breaks and is probably nasty
     if (isSpecialWindow() || noBorder())
         mode = ShadeNone;
     mode = rules()->checkShade(mode);
@@ -1117,19 +1099,35 @@ void Client::setShade(ShadeMode mode)
         XSelectInput(display(), wrapper, ClientWinMask | SubstructureNotifyMask);
         plainResize(s);
         shade_geometry_change = false;
-        if (isActive()) {
-            if (was_shade_mode == ShadeHover)
+        if (was_shade_mode == ShadeHover) {
+            if (shade_below && workspace()->stackingOrder().indexOf(shade_below) > -1)
+                    workspace()->restack(this, shade_below);
+            if (isActive())
                 workspace()->activateNextClient(this);
-            else
-                workspace()->focusToNull();
+        } else if (isActive()) {
+            workspace()->focusToNull();
         }
     } else {
         shade_geometry_change = true;
         QSize s(sizeForClientSize(clientSize()));
         shade_geometry_change = false;
         plainResize(s);
-        if (shade_mode == ShadeHover || shade_mode == ShadeActivated)
+        if ((shade_mode == ShadeHover || shade_mode == ShadeActivated) && rules()->checkAcceptFocus(input))
             setActive(true);
+        if (shade_mode == ShadeHover) {
+            ToplevelList order = workspace()->stackingOrder();
+            // this is likely related to the index parameter?!
+            for (int idx = order.indexOf(this) + 1; idx < order.count(); ++idx) {
+                shade_below = qobject_cast<Client*>(order.at(idx));
+                if (shade_below) {
+                    break;
+                }
+            }
+            if (shade_below && shade_below->isNormalWindow())
+                workspace()->raiseClient(this);
+            else
+                shade_below = NULL;
+        }
         XMapWindow(display(), wrapperId());
         XMapWindow(display(), window());
         if (isActive())
@@ -1142,11 +1140,12 @@ void Client::setShade(ShadeMode mode)
     updateAllowedActions();
     if (decoration)
         decoration->shadeChange();
-    updateWindowRules();
+    updateWindowRules(Rules::Shade);
 
     // Update states of all other windows in this group
-    if (clientGroup())
-        clientGroup()->updateStates(this);
+    if (tabGroup())
+        tabGroup()->updateStates(this, TabGroup::Shaded);
+    emit shadeChanged();
 }
 
 void Client::shadeHover()
@@ -1177,20 +1176,20 @@ void Client::updateVisibility()
 {
     if (deleting)
         return;
-    if (hidden && (clientGroup() == NULL || clientGroup()->visible() == this)) {
+    if (hidden && isCurrentTab()) {
         info->setState(NET::Hidden, NET::Hidden);
         setSkipTaskbar(true, false);   // Also hide from taskbar
-        if (compositing() && options->hiddenPreviews == HiddenPreviewsAlways)
+        if (compositing() && options->hiddenPreviews() == HiddenPreviewsAlways)
             internalKeep(Allowed);
         else
             internalHide(Allowed);
         return;
     }
-    if (clientGroup() == NULL || clientGroup()->visible() == this)
+    if (isCurrentTab())
         setSkipTaskbar(original_skip_taskbar, false);   // Reset from 'hidden'
     if (minimized) {
         info->setState(NET::Hidden, NET::Hidden);
-        if (compositing() && options->hiddenPreviews == HiddenPreviewsAlways)
+        if (compositing() && options->hiddenPreviews() == HiddenPreviewsAlways)
             internalKeep(Allowed);
         else
             internalHide(Allowed);
@@ -1198,14 +1197,14 @@ void Client::updateVisibility()
     }
     info->setState(0, NET::Hidden);
     if (!isOnCurrentDesktop()) {
-        if (compositing() && options->hiddenPreviews != HiddenPreviewsNever)
+        if (compositing() && options->hiddenPreviews() != HiddenPreviewsNever)
             internalKeep(Allowed);
         else
             internalHide(Allowed);
         return;
     }
     if (!isOnCurrentActivity()) {
-        if (compositing() && options->hiddenPreviews != HiddenPreviewsNever)
+        if (compositing() && options->hiddenPreviews() != HiddenPreviewsNever)
             internalKeep(Allowed);
         else
             internalHide(Allowed);
@@ -1444,14 +1443,14 @@ void Client::pingWindow()
 {
     if (!Pping)
         return; // Can't ping :(
-    if (options->killPingTimeout == 0)
+    if (options->killPingTimeout() == 0)
         return; // Turned off
     if (ping_timer != NULL)
         return; // Pinging already
     ping_timer = new QTimer(this);
     connect(ping_timer, SIGNAL(timeout()), SLOT(pingTimeout()));
     ping_timer->setSingleShot(true);
-    ping_timer->start(options->killPingTimeout);
+    ping_timer->start(options->killPingTimeout());
     ping_timestamp = xTime();
     workspace()->sendPingToWindow(window(), ping_timestamp);
 }
@@ -1463,13 +1462,9 @@ void Client::gotPing(Time timestamp)
         return;
     delete ping_timer;
     ping_timer = NULL;
-    if (process_killer != NULL) {
-        process_killer->kill();
-        // Recycle when the process manager has noticed that the process exited
-        // a delete process_killer here sometimes causes a hang in waitForFinished
-        connect(process_killer, SIGNAL(finished(int,QProcess::ExitStatus)),
-                process_killer, SLOT(deleteLater()));
-        process_killer = NULL;
+    if (m_killHelperPID && !::kill(m_killHelperPID, 0)) { // means the process is alive
+        ::kill(m_killHelperPID, SIGTERM);
+        m_killHelperPID = 0;
     }
 }
 
@@ -1483,7 +1478,7 @@ void Client::pingTimeout()
 
 void Client::killProcess(bool ask, Time timestamp)
 {
-    if (process_killer != NULL)
+    if (m_killHelperPID && !::kill(m_killHelperPID, 0)) // means the process is alive
         return;
     Q_ASSERT(!ask || timestamp != CurrentTime);
     QByteArray machine = wmClientMachine(true);
@@ -1499,25 +1494,16 @@ void Client::killProcess(bool ask, Time timestamp)
         } else
             ::kill(pid, SIGTERM);
     } else {
-        process_killer = new QProcess(this);
-        connect(process_killer, SIGNAL(error(QProcess::ProcessError)), SLOT(processKillerExited()));
-        connect(process_killer, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(processKillerExited()));
-        process_killer->start(KStandardDirs::findExe("kwin_killer_helper"),
-                              QStringList() << "--pid" << QByteArray().setNum(unsigned(pid)) << "--hostname" << machine
-                              << "--windowname" << caption()
-                              << "--applicationname" << resourceClass()
-                              << "--wid" << QString::number(window())
-                              << "--timestamp" << QString::number(timestamp));
+        QProcess::startDetached(KStandardDirs::findExe("kwin_killer_helper"),
+                                QStringList() << "--pid" << QByteArray().setNum(unsigned(pid)) << "--hostname" << machine
+                                << "--windowname" << caption()
+                                << "--applicationname" << resourceClass()
+                                << "--wid" << QString::number(window())
+                                << "--timestamp" << QString::number(timestamp),
+                                QString(), &m_killHelperPID);
     }
 }
 
-void Client::processKillerExited()
-{
-    kDebug(1212) << "Killer exited";
-    delete process_killer;
-    process_killer = NULL;
-}
-
 void Client::setSkipTaskbar(bool b, bool from_outside)
 {
     int was_wants_tab_focus = wantsTabFocus();
@@ -1529,10 +1515,11 @@ void Client::setSkipTaskbar(bool b, bool from_outside)
         return;
     skip_taskbar = b;
     info->setState(b ? NET::SkipTaskbar : 0, NET::SkipTaskbar);
-    updateWindowRules();
+    updateWindowRules(Rules::SkipTaskbar);
     if (was_wants_tab_focus != wantsTabFocus())
         workspace()->updateFocusChains(this,
                                        isActive() ? Workspace::FocusChainMakeFirst : Workspace::FocusChainUpdate);
+    emit skipTaskbarChanged();
 }
 
 void Client::setSkipPager(bool b)
@@ -1542,7 +1529,8 @@ void Client::setSkipPager(bool b)
         return;
     skip_pager = b;
     info->setState(b ? NET::SkipPager : 0, NET::SkipPager);
-    updateWindowRules();
+    updateWindowRules(Rules::SkipPager);
+    emit skipPagerChanged();
 }
 
 void Client::setSkipSwitcher(bool set)
@@ -1551,7 +1539,8 @@ void Client::setSkipSwitcher(bool set)
     if (set == skipSwitcher())
         return;
     skip_switcher = set;
-    updateWindowRules();
+    updateWindowRules(Rules::SkipSwitcher);
+    emit skipSwitcherChanged();
 }
 
 void Client::setModal(bool m)
@@ -1560,8 +1549,7 @@ void Client::setModal(bool m)
     if (modal == m)
         return;
     modal = m;
-    if (!modal)
-        return;
+    emit modalChanged();
     // Changing modality for a mapped window is weird (?)
     // _NET_WM_STATE_MODAL should possibly rather be _NET_WM_WINDOW_TYPE_MODAL_DIALOG
 }
@@ -1602,11 +1590,12 @@ void Client::setDesktop(int desktop)
 
     workspace()->updateFocusChains(this, Workspace::FocusChainMakeFirst);
     updateVisibility();
-    updateWindowRules();
+    updateWindowRules(Rules::Desktop);
 
     // Update states of all other windows in this group
-    if (clientGroup())
-        clientGroup()->updateStates(this);
+    if (tabGroup())
+        tabGroup()->updateStates(this, TabGroup::Desktop);
+    emit desktopChanged();
 }
 
 /**
@@ -1636,18 +1625,26 @@ void Client::setOnActivity(const QString &activity, bool enable)
  */
 void Client::setOnActivities(QStringList newActivitiesList)
 {
+    QString joinedActivitiesList = newActivitiesList.join(",");
+    joinedActivitiesList = rules()->checkActivity(joinedActivitiesList, false);
+    newActivitiesList = joinedActivitiesList.split(',', QString::SkipEmptyParts);
+
     QStringList allActivities = workspace()->activityList();
     if (newActivitiesList.size() == allActivities.size() || newActivitiesList.isEmpty()) {
         setOnAllActivities(true);
-        return;
-    }
+        activityList.clear();
+        XChangeProperty(display(), window(), atoms->activities, XA_STRING, 8,
+                        PropModeReplace, (const unsigned char *)"ALL", 3);
 
-    QByteArray joined = newActivitiesList.join(",").toAscii();
-    char *data = joined.data();
-    activityList = newActivitiesList;
-    XChangeProperty(display(), window(), atoms->activities, XA_STRING, 8,
+    } else {
+        QByteArray joined = joinedActivitiesList.toAscii();
+        char *data = joined.data();
+        activityList = newActivitiesList;
+        XChangeProperty(display(), window(), atoms->activities, XA_STRING, 8,
                     PropModeReplace, (unsigned char *)data, joined.size());
 
+    }
+
     updateActivities(false);
 }
 
@@ -1664,11 +1661,11 @@ void Client::updateActivities(bool includeTransients)
         workspace()->updateOnAllActivitiesOfTransients(this);
     workspace()->updateFocusChains(this, Workspace::FocusChainMakeFirst);
     updateVisibility();
-    // TODO: add activity rule
+    updateWindowRules(Rules::Activity);
 
     // Update states of all other windows in this group
-    if (clientGroup())
-        clientGroup()->updateStates(this);
+    if (tabGroup())
+        tabGroup()->updateStates(this, TabGroup::Activity);
 }
 
 /**
@@ -1709,8 +1706,8 @@ void Client::setOnAllDesktops(bool b)
         setDesktop(workspace()->currentDesktop());
 
     // Update states of all other windows in this group
-    if (clientGroup())
-        clientGroup()->updateStates(this);
+    if (tabGroup())
+        tabGroup()->updateStates(this, TabGroup::Desktop);
 }
 
 /**
@@ -1722,10 +1719,8 @@ void Client::setOnAllActivities(bool on)
     if (on == isOnAllActivities())
         return;
     if (on) {
-        activityList.clear();
-        XChangeProperty(display(), window(), atoms->activities, XA_STRING, 8,
-                        PropModeReplace, (const unsigned char *)"ALL", 3);
-        updateActivities(true);
+        setOnActivities(QStringList());
+
     } else {
         setOnActivity(Workspace::self()->currentActivity(), true);
         workspace()->updateOnAllActivitiesOfTransients(this);
@@ -1782,6 +1777,8 @@ void Client::takeFocus(allowed_t)
 #endif
     if (rules()->checkAcceptFocus(input))
         XSetInputFocus(display(), window(), RevertToPointerRoot, xTime());
+    else
+        demandAttention(false); // window cannot take input, at least withdraw urgency
     if (Ptakefocus)
         sendClientMessage(window(), atoms->wm_protocols, atoms->wm_take_focus);
     workspace()->setShouldGetFocus(this);
@@ -1873,11 +1870,10 @@ void Client::setCaption(const QString& _s, bool force)
             // Keep the same suffix in iconic name if it's set
             info->setVisibleIconName(QString(cap_iconic + cap_suffix).toUtf8());
 
-        if (isManaged() && decoration != NULL) {
-            if (client_group)
-                client_group->updateItems();
+        if (isManaged() && decoration) {
             decoration->captionChange();
         }
+        emit captionChanged();
     }
 }
 
@@ -1913,12 +1909,68 @@ QString Client::caption(bool full) const
     return full ? cap_normal + cap_suffix : cap_normal;
 }
 
-void Client::setClientGroup(ClientGroup* group)
+bool Client::tabTo(Client *other, bool behind, bool activate)
 {
-    client_group = group;
-    unsigned long data[1] = {(unsigned long)workspace()->indexOfClientGroup(group)};
-    XChangeProperty(display(), window(), atoms->kde_net_wm_tab_group, XA_CARDINAL, 32,
-                    PropModeReplace, (unsigned char*)(data), 1);
+    Q_ASSERT(other && other != this);
+
+    if (tab_group && tab_group == other->tabGroup()) { // special case: move inside group
+        tab_group->move(this, other, behind);
+        return true;
+    }
+
+    GeometryUpdatesBlocker blocker(this);
+    const bool wasBlocking = signalsBlocked();
+    blockSignals(true); // prevent client emitting "retabbed to nowhere" cause it's about to be entabbed the next moment
+    untab();
+    blockSignals(wasBlocking);
+
+    TabGroup *newGroup = other->tabGroup() ? other->tabGroup() : new TabGroup(other);
+
+    if (!newGroup->add(this, other, behind, activate)) {
+        if (newGroup->count() < 2) { // adding "c" to "to" failed for whatever reason
+            newGroup->remove(other);
+            delete newGroup;
+        }
+        return false;
+    }
+    return true;
+}
+
+bool Client::untab(const QRect &toGeometry)
+{
+    TabGroup *group = tab_group;
+    if (group && group->remove(this, toGeometry)) { // remove sets the tabgroup to "0", therefore the pointer is cached
+        if (group->isEmpty()) {
+            delete group;
+        }
+        setClientShown(!(isMinimized() || isShade()));
+        return true;
+    }
+    return false;
+}
+
+void Client::setTabGroup(TabGroup *group)
+{
+    tab_group = group;
+    if (group) {
+        unsigned long data = qHash(group); //->id();
+        XChangeProperty(display(), window(), atoms->kde_net_wm_tab_group, XA_CARDINAL, 32,
+                        PropModeReplace, (unsigned char*)(&data), 1);
+    }
+    else
+        XDeleteProperty(display(), window(), atoms->kde_net_wm_tab_group);
+    emit tabGroupChanged();
+}
+
+bool Client::isCurrentTab() const
+{
+    return !tab_group || tab_group->current() == this;
+}
+
+void Client::syncTabGroupFor(QString property, bool fromThisClient)
+{
+    if (tab_group)
+        tab_group->sync(property.toAscii().data(), fromThisClient ? this : tab_group->current());
 }
 
 void Client::dontMoveResize()
@@ -1933,26 +1985,20 @@ void Client::setClientShown(bool shown)
 {
     if (deleting)
         return; // Don't change shown status if this client is being deleted
-    if (shown && hidden) {
+    if (shown != hidden)
+        return; // nothing to change
+    hidden = !shown;
+    if (options->isInactiveTabsSkipTaskbar())
+        setSkipTaskbar(hidden, false); // TODO: Causes reshuffle of the taskbar
+    if (shown) {
         map(Allowed);
-        hidden = false;
-        //updateVisibility();
-        //updateAllowedActions();
-        if (options->inactiveTabsSkipTaskbar)
-            setSkipTaskbar(false, false);
         takeFocus(Allowed);
         autoRaise();
         workspace()->updateFocusChains(this, Workspace::FocusChainMakeFirst);
-    }
-    if (!shown && !hidden) {
+    } else {
         unmap(Allowed);
-        hidden = true;
-        //updateVisibility();
-        //updateAllowedActions();
-        if (options->inactiveTabsSkipTaskbar)
-            setSkipTaskbar(true, false);   // TODO: Causes reshuffle of the taskbar
         // Don't move tabs to the end of the list when another tab get's activated
-        if (!clientGroup() || clientGroup()->visible() == this)
+        if (isCurrentTab())
             workspace()->updateFocusChains(this, Workspace::FocusChainMakeLast);
         addWorkspaceRepaint(visibleRect());
     }
@@ -1977,11 +2023,6 @@ void Client::getWMHints()
     updateAllowedActions(); // Group affects isMinimizable()
 }
 
-void Client::sl_activated()
-{
-    emit s_activated();
-}
-
 void Client::getMotifHints()
 {
     bool mgot_noborder, mnoborder, mresize, mmove, mminimize, mmaximize, mclose;
@@ -2070,6 +2111,7 @@ void Client::getIcons()
     }
     if (isManaged() && decoration != NULL)
         decoration->iconChange();
+    emit iconChanged();
 }
 
 QPixmap Client::icon(const QSize& size) const
@@ -2199,14 +2241,15 @@ void Client::sendSyncRequest()
 void Client::removeSyncSupport()
 {
     if (!ready_for_painting) {
-        ready_for_painting = true;
-        addRepaintFull();
+        setReadyForPainting();
         return;
     }
+#ifdef HAVE_XSYNC
     syncRequest.isPending = false;
     syncRequest.counter = syncRequest.alarm = None;
     delete syncRequest.timeout; delete syncRequest.failsafeTimeout;
     syncRequest.timeout = syncRequest.failsafeTimeout = NULL;
+#endif
 }
 
 bool Client::wantsTabFocus() const
@@ -2274,16 +2317,23 @@ void Client::updateCursor()
 
 void Client::updateCompositeBlocking(bool readProperty)
 {
-    const bool usedToBlock = blocks_compositing;
     if (readProperty) {
         const unsigned long properties[2] = {0, NET::WM2BlockCompositing};
         NETWinInfo2 i(QX11Info::display(), window(), rootWindow(), properties, 2);
-        blocks_compositing = rules()->checkBlockCompositing(i.isBlockingCompositing());
+        setBlockingCompositing(i.isBlockingCompositing());
     }
     else
-        blocks_compositing = rules()->checkBlockCompositing(blocks_compositing);
-    if (usedToBlock != blocks_compositing)
+        setBlockingCompositing(blocks_compositing);
+}
+
+void Client::setBlockingCompositing(bool block)
+{
+    const bool usedToBlock = blocks_compositing;
+    blocks_compositing = rules()->checkBlockCompositing(block);
+    if (usedToBlock != blocks_compositing) {
         workspace()->updateCompositeBlocking(blocks_compositing ? this : 0);
+        emit blockingCompositingChanged();
+    }
 }
 
 Client::Position Client::mousePosition(const QPoint& p) const
@@ -2408,6 +2458,29 @@ QRect Client::decorationRect() const
     }
 }
 
+void Client::updateFirstInTabBox()
+{
+    // TODO: move into KWindowInfo
+    Atom type;
+    int format, status;
+    unsigned long nitems = 0;
+    unsigned long extra = 0;
+    unsigned char *data = 0;
+    status = XGetWindowProperty(display(), window(), atoms->kde_first_in_window_list, 0, 1, false, atoms->kde_first_in_window_list, &type, &format, &nitems, &extra, &data);
+    if (status == Success && format == 32 && nitems == 1) {
+        setFirstInTabBox(true);
+    } else {
+        setFirstInTabBox(false);
+    }
+    if (data)
+        XFree(data);
+}
+
+bool Client::isClient() const
+{
+    return true;
+}
+
 } // namespace
 
 #include "client.moc"
