@@ -37,6 +37,7 @@
 #include <KLocale>
 #include <KColorUtils>
 #include <KDebug>
+#include <KStyle>
 
 #include <QtGui/QApplication>
 #include <QtGui/QLabel>
@@ -118,6 +119,10 @@ namespace Oxygen
             foreach( QLabel* widget, children )
             { widget->setAutoFillBackground( false ); }
 
+            // also change shadow configuration size to something that fits in the preview list
+            shadowCache().setShadowSize( QPalette::Active, 15 );
+            shadowCache().setShadowSize( QPalette::Inactive, 15 );
+
         }
 
         _initialized = true;
@@ -507,7 +512,7 @@ namespace Oxygen
         const QRect titleRect( this->titleRect() );
 
         // get tabs
-        const int items( clientGroupItems().count() );
+        const int items( tabCount() );
 
         // make sure item data have the correct number of items
         while( _itemData.count() < items ) _itemData.push_back( ClientGroupItemData() );
@@ -570,7 +575,7 @@ namespace Oxygen
         }
 
         // button activity
-        _itemData.updateButtonActivity( visibleClientGroupItem() );
+        _itemData.updateButtonActivity( currentTabId() );
 
         // reset buttons location
         _itemData.updateButtons( alsoUpdate );
@@ -609,13 +614,8 @@ namespace Oxygen
             int offset = layoutMetric( LM_OuterPaddingTop );
 
             // radial gradient positionning
-            int height = 64 - Configuration::ButtonDefault;
-            if( !hideTitleBar() ) height += configuration().buttonSize();
-            if( isMaximized() )
-            {
-                offset -= 3;
-                height -= 3;
-            }
+            const int height = hideTitleBar() ? 0:configuration().buttonSize();
+            if( isMaximized() ) offset -= 3;
 
             const QWidget* window( isPreview() ? this->widget() : widget->window() );
             helper().renderWindowBackground(painter, rect, widget, window, palette, offset, height );
@@ -628,8 +628,7 @@ namespace Oxygen
             int offset = layoutMetric( LM_OuterPaddingTop );
 
             // radial gradient positionning
-            int height = 64 - Configuration::ButtonDefault;
-            if( !hideTitleBar() ) height += configuration().buttonSize();
+            const int height = hideTitleBar() ? 0:configuration().buttonSize();
             if( isMaximized() ) offset -= 3;
 
             // background pixmap
@@ -681,7 +680,7 @@ namespace Oxygen
         const int titleHeight( layoutMetric( LM_TitleEdgeTop ) + layoutMetric( LM_TitleEdgeBottom ) + layoutMetric( LM_TitleHeight ) );
 
         // make titlebar background darker for tabbed, non-outline window
-        if( ( clientGroupItems().count() >= 2 || _itemData.isAnimated() ) && !(configuration().drawTitleOutline() && isActive() ) )
+        if( ( tabCount() >= 2 || _itemData.isAnimated() ) && !(configuration().drawTitleOutline() && isActive() ) )
         {
 
             const QPoint topLeft( r.topLeft()-position );
@@ -938,7 +937,6 @@ namespace Oxygen
             if( isMaximized() ) painter->translate( 0, -2 );
 
         } else if( !caption().isEmpty() ) {
-
             renderTitleText( painter, rect, caption(), color, contrast );
 
         }
@@ -1017,11 +1015,10 @@ namespace Oxygen
         { textRect.adjust( 0, 0, - configuration().buttonSize() - layoutMetric(LM_TitleEdgeRight), 0 ); }
 
         // check if current item is active
-        const bool active( index == visibleClientGroupItem() );
+        const bool active( tabId(index) == currentTabId() );
 
         // get current item caption and update text rect
-        const QList< ClientGroupItem >& items( clientGroupItems() );
-        const QString caption( itemCount == 1 ? this->caption() : items[index].title() );
+        const QString caption( itemCount == 1 ? this->caption() : this->caption(index) );
 
         if( !configuration().centerTitleOnFullWidth() )
         { boundRectTo( textRect, titleRect() ); }
@@ -1108,7 +1105,7 @@ namespace Oxygen
             if(
                 ( index == itemCount-1 && buttonsRightWidth() > 0 ) ||
                 ( index+1 < itemCount && ( _itemData.isTarget( index+1 ) ||
-                !( index+1 == visibleClientGroupItem() && _itemData[index+1]._boundingRect.isValid() ) ) ) )
+                !( tabId(index+1) == currentTabId() && _itemData[index+1]._boundingRect.isValid() ) ) ) )
             {
 
                 const QRect local( item._boundingRect.topRight()+QPoint(0,2), QSize( 2, item._boundingRect.height()-3 ) );
@@ -1554,12 +1551,12 @@ namespace Oxygen
 
         }
 
-        // make sure ItemData and clientGroupItems are synchronized
+        // make sure ItemData and tabList are synchronized
         /*
         this needs to be done before calling RenderWindowBorder
         since some painting in there depend on the clientGroups state
         */
-        if(  _itemData.isDirty() || _itemData.count() != clientGroupItems().count() )
+        if(  _itemData.isDirty() || _itemData.count() != tabCount() )
         { updateItemBoundingRects( false ); }
 
         // window background
@@ -1607,12 +1604,12 @@ namespace Oxygen
     {
 
         const QPoint point = event->pos();
-        if( itemClicked( point ) < 0 ) return false;
+        if( tabIndexAt( point ) < 0 ) return false;
         _dragPoint = point;
 
         _mouseButton = event->button();
         bool accepted( false );
-        if( buttonToWindowOperation( _mouseButton ) == ClientGroupDragOp )
+        if( buttonToWindowOperation( _mouseButton ) == TabDragOp )
         {
 
             accepted = true;
@@ -1620,9 +1617,10 @@ namespace Oxygen
         } else if( buttonToWindowOperation( _mouseButton ) == OperationsOp ) {
 
             QPoint point = event->pos();
-            int itemClicked( this->itemClicked( point ) );
+            const int clickedIndex( tabIndexAt( point ) );
             _mouseButton = Qt::NoButton;
-            displayClientMenu( itemClicked, widget()->mapToGlobal( event->pos() ) );
+            if ( tabIndexAt( point ) > -1)
+                showWindowMenu( widget()->mapToGlobal( event->pos() ), tabId(clickedIndex) );
             accepted = true; // displayClientMenu can possibly destroy the deco...
 
         }
@@ -1640,11 +1638,11 @@ namespace Oxygen
 
             const QPoint point = event->pos();
 
-            const int visibleItem = visibleClientGroupItem();
-            const int itemClicked( this->itemClicked( point ) );
-            if( itemClicked >= 0 && visibleItem != itemClicked )
+            const long visibleItem = currentTabId();
+            const int clickedIndex( tabIndexAt( point ) );
+            if( clickedIndex >= 0 && visibleItem != tabId(clickedIndex) )
             {
-                setVisibleClientGroupItem( itemClicked );
+                setCurrentTab( tabId(clickedIndex) );
                 setForceActive( true );
                 accepted = true;
             }
@@ -1665,23 +1663,23 @@ namespace Oxygen
         { return false; }
 
         bool accepted( false );
-        if( buttonToWindowOperation( _mouseButton ) == ClientGroupDragOp )
+        if( buttonToWindowOperation( _mouseButton ) == TabDragOp )
         {
 
             const QPoint point = event->pos();
-            const int itemClicked( this->itemClicked( point ) );
-            if( itemClicked < 0 ) return false;
+            const int clickedIndex( tabIndexAt( point ) );
+            if( clickedIndex < 0 ) return false;
 
             _titleAnimationData->reset();
 
             QDrag *drag = new QDrag( widget() );
             QMimeData *groupData = new QMimeData();
-            groupData->setData( clientGroupItemDragMimeType(), QString().setNum( itemId( itemClicked )).toAscii() );
+            groupData->setData( tabDragMimeType(), QString().setNum( tabId(clickedIndex) ).toAscii() );
             drag->setMimeData( groupData );
-            _sourceItem = this->itemClicked( _dragPoint );
+            _sourceItem = tabIndexAt( _dragPoint );
 
             // get tab geometry
-            QRect geometry( _itemData[itemClicked]._boundingRect );
+            QRect geometry( _itemData[clickedIndex]._boundingRect );
 
             // remove space used for buttons
             if( _itemData.count() > 1  )
@@ -1697,11 +1695,23 @@ namespace Oxygen
 
             }
 
-            drag->setPixmap( itemDragPixmap( itemClicked, geometry ) );
+            // adjust geometry to include shadow size
+            const int shadowSize( shadowCache().shadowSize() );
+            const bool drawShadow(
+                compositingActive() &&
+                KStyle::customStyleHint( "SH_ArgbDndWindow", widget() ) &&
+                shadowSize > 0 );
+
+            if( drawShadow )
+            { geometry.adjust( -shadowSize, -shadowSize, shadowSize, shadowSize ); }
+
+            // compute pixmap and assign
+            drag->setPixmap( itemDragPixmap( clickedIndex, geometry, drawShadow ) );
 
             // note: the pixmap is moved just above the pointer on purpose
             // because overlapping pixmap and pointer slows down the pixmap a lot.
             QPoint hotSpot( QPoint( event->pos().x() - geometry.left(), -1 ) );
+            if( drawShadow ) hotSpot += QPoint( 0, shadowSize );
 
             // make sure the horizontal hotspot position is not too far away (more than 1px)
             // from the pixmap
@@ -1717,7 +1727,7 @@ namespace Oxygen
             if( drag->target() == 0 && _itemData.count() > 1 )
             {
                 _itemData.setDirty( true );
-                removeFromClientGroup( _sourceItem,
+                untab( tabId(_sourceItem),
                     widget()->frameGeometry().adjusted(
                     layoutMetric( LM_OuterPaddingLeft ),
                     layoutMetric( LM_OuterPaddingTop ),
@@ -1743,7 +1753,7 @@ namespace Oxygen
     {
 
         // check if drag enter is allowed
-        if( !event->mimeData()->hasFormat( clientGroupItemDragMimeType() ) || hideTitleBar() ) return false;
+        if( !event->mimeData()->hasFormat( tabDragMimeType() ) || hideTitleBar() ) return false;
 
         //
         event->acceptProposedAction();
@@ -1751,13 +1761,13 @@ namespace Oxygen
         {
 
             const QPoint position( event->pos() );
-            _itemData.animate( AnimationEnter, itemClicked( position, true ) );
+            _itemData.animate( AnimationEnter, tabIndexAt( position, true ) );
 
         } else if( _itemData.count() > 1 )  {
 
             const QPoint position( event->pos() );
-            const int itemClicked( this->itemClicked( position, false ) );
-            _itemData.animate( AnimationEnter|AnimationSameTarget, itemClicked );
+            const int clickedIndex( tabIndexAt( position, false ) );
+            _itemData.animate( AnimationEnter|AnimationSameTarget, clickedIndex );
 
         }
 
@@ -1791,20 +1801,20 @@ namespace Oxygen
     {
 
         // check format
-        if( !event->mimeData()->hasFormat( clientGroupItemDragMimeType() ) ) return false;
+        if( !event->mimeData()->hasFormat( tabDragMimeType() ) ) return false;
         if( event->source() != widget() )
         {
 
             const QPoint position( event->pos() );
-            _itemData.animate( AnimationMove, itemClicked( position, true ) );
+            _itemData.animate( AnimationMove, tabIndexAt( position, true ) );
 
         } else if( _itemData.count() > 1 )  {
 
             if( _dragStartTimer.isActive() ) _dragStartTimer.stop();
 
             const QPoint position( event->pos() );
-            const int itemClicked( this->itemClicked( position, false ) );
-            _itemData.animate( AnimationMove|AnimationSameTarget, itemClicked );
+            const int clickedIndex( tabIndexAt( position, false ) );
+            _itemData.animate( AnimationMove|AnimationSameTarget, clickedIndex );
 
         }
 
@@ -1820,35 +1830,25 @@ namespace Oxygen
         _itemData.animate( AnimationNone );
 
         const QMimeData *groupData = event->mimeData();
-        if( !groupData->hasFormat( clientGroupItemDragMimeType() ) ) return false;
+        if( !groupData->hasFormat( tabDragMimeType() ) ) return false;
+
+        _itemData.setDirty( true );
+
+        if( widget() != event->source() )
+            setForceActive( true );
+
+        const long source = QString( groupData->data( tabDragMimeType() ) ).toLong();
+        int clickedIndex( tabIndexAt( point, true ) );
+        if (clickedIndex < 0)
+            tab_A_behind_B(source, tabId(_itemData.count() - 1));
+        else if (clickedIndex)
+            tab_A_behind_B(source, tabId(clickedIndex));
+        else
+            tab_A_before_B(source, tabId(clickedIndex));
 
         if( widget() == event->source() )
-        {
-
-            const int from = this->itemClicked( _dragPoint );
-            int itemClicked( this->itemClicked( point, false ) );
-
-            if( itemClicked > from )
-            {
-                itemClicked++;
-                if( itemClicked >= clientGroupItems().count() )
-                { itemClicked = -1; }
-            }
-
-            _itemData.setDirty( true );
-            moveItemInClientGroup( from, itemClicked );
             updateTitleRect();
 
-        } else {
-
-            setForceActive( true );
-            const int itemClicked( this->itemClicked( point, true ) );
-            const long source = QString( groupData->data( clientGroupItemDragMimeType() ) ).toLong();
-            _itemData.setDirty( true );
-            moveItemToClientGroup( source, itemClicked );
-
-        }
-
         _titleAnimationData->reset();
         return true;
 
@@ -1881,7 +1881,7 @@ namespace Oxygen
             if( button == _itemData[i]._closeButton.data() )
             {
                 _itemData.setDirty( true );
-                closeClientGroupItem( i );
+                closeTab( tabId(i) );
                 return true;
             }
         }
@@ -1890,21 +1890,39 @@ namespace Oxygen
     }
 
     //________________________________________________________________
-    QPixmap Client::itemDragPixmap( int index, const QRect& geometry )
+    QPixmap Client::itemDragPixmap( int index, QRect geometry, bool drawShadow )
     {
-        const bool itemValid( index >= 0 && index < clientGroupItems().count() );
+        const bool itemValid( index >= 0 && index < tabCount() );
 
         QPixmap pixmap( geometry.size() );
+        pixmap.fill( Qt::transparent );
         QPainter painter( &pixmap );
         painter.setRenderHints(QPainter::SmoothPixmapTransform|QPainter::Antialiasing);
 
         painter.translate( -geometry.topLeft() );
 
+        // draw shadows
+        if( drawShadow )
+        {
+
+            // shadow
+            const int shadowSize( shadowCache().shadowSize() );
+            TileSet *tileSet( shadowCache().tileSet( ShadowCache::Key() ) );
+            tileSet->render( geometry, &painter, TileSet::Ring);
+            geometry.adjust( shadowSize, shadowSize, -shadowSize, -shadowSize );
+
+            renderCorners( &painter, geometry, widget()->palette() );
+
+        }
+
+        // mask
+        painter.setClipRegion( helper().roundedMask( geometry ), Qt::IntersectClip );
+
         // render window background
         renderWindowBackground( &painter, geometry, widget(), widget()->palette() );
 
         // darken background if item is inactive
-        const bool itemActive = !( itemValid && index != visibleClientGroupItem() );
+        const bool itemActive = (tabCount() <= 1) || !( itemValid && tabId(index) != currentTabId() );
         if( !itemActive )
         {
 
@@ -1924,32 +1942,25 @@ namespace Oxygen
         if( itemValid )
         { textRect.adjust( layoutMetric( LM_TitleBorderLeft ), 0, -layoutMetric(LM_TitleBorderRight), 0 ); }
 
-        const QString caption( itemValid ? clientGroupItems()[index].title() : this->caption() );
+        const QString caption( itemValid ? this->caption(index) : this->caption() );
+
         renderTitleText(
             &painter, textRect, caption,
             titlebarTextColor( widget()->palette(), isActive() && itemActive ),
             titlebarContrastColor( widget()->palette() ) );
 
+        // adjust geometry for floatFrame when compositing is on.
+        if( drawShadow )
+        { geometry.adjust(-1, -1, 1, 1 ); }
+
         // floating frame
         helper().drawFloatFrame(
             &painter, geometry, widget()->palette().window().color(),
-            true, false,
+            !drawShadow, false,
             KDecoration::options()->color(ColorTitleBar)
             );
 
         painter.end();
-
-        // create pixmap mask
-        QBitmap bitmap( geometry.size() );
-        {
-            bitmap.clear();
-            QPainter painter( &bitmap );
-            QPainterPath path;
-            path.addRegion( helper().roundedMask( geometry.translated( -geometry.topLeft() ) ) );
-            painter.fillPath( path, Qt::color1 );
-        }
-
-        pixmap.setMask( bitmap );
         return pixmap;
 
     }
