@@ -35,6 +35,7 @@ Deleted::Deleted(Workspace* ws)
     , padding_top(0)
     , padding_right(0)
     , padding_bottom(0)
+    , m_layer(UnknownLayer)
 {
 }
 
@@ -51,7 +52,7 @@ Deleted* Deleted::create(Toplevel* c)
 {
     Deleted* d = new Deleted(c->workspace());
     d->copyToDeleted(c);
-    d->workspace()->addDeleted(d, Allowed);
+    d->workspace()->addDeleted(d, c, Allowed);
     return d;
 }
 
@@ -66,19 +67,11 @@ void Deleted::copyToDeleted(Toplevel* c)
 {
     assert(dynamic_cast< Deleted* >(c) == NULL);
     Toplevel::copyToDeleted(c);
-    // In some cases the window has been deleted before the sync request which marks
-    // the window ready for painting has finished. This is especially troublesome
-    // when effects reference the deleted window and the unreferencing is part of
-    // the rendering pass (e.g. Effect::postPaintScreen/postPaintWindow), which will
-    // never be executed because we remove it every time from the stacking list in
-    // Workspace::performCompositing.
-    if (!c->readyForPainting()) {
-        QTimer::singleShot(0, this, SLOT(discard()));
-    }
     desk = c->desktop();
     activityList = c->activities();
     contentsRect = QRect(c->clientPos(), c->clientSize());
     transparent_rect = c->transparentRect();
+    m_layer = c->layer();
     if (WinInfo* cinfo = dynamic_cast< WinInfo* >(info))
         cinfo->disable();
     Client* client = dynamic_cast<Client*>(c);
@@ -157,6 +150,11 @@ QRect Deleted::transparentRect() const
     return transparent_rect;
 }
 
+bool Deleted::isDeleted() const
+{
+    return true;
+}
+
 } // namespace
 
 #include "deleted.moc"
