@@ -35,14 +35,13 @@ public:
     // used by effects to keep the window around for e.g. fadeout effects when it's destroyed
     void refWindow();
     void unrefWindow(bool delay = false);
+    void discard(allowed_t);
     virtual int desktop() const;
     virtual QStringList activities() const;
     virtual QPoint clientPos() const;
     virtual QSize clientSize() const;
     virtual QRect transparentRect() const;
-    virtual inline QRect visibleRect() const {
-        return Toplevel::visibleRect().adjusted(-padding_left, -padding_top, padding_right, padding_bottom);
-    }
+    virtual bool isDeleted() const;
     const QPixmap *topDecoPixmap() const {
         return &decorationPixmapTop;
     }
@@ -60,8 +59,9 @@ public:
     }
     void layoutDecorationRects(QRect &left, QRect &top, QRect &right, QRect &bottom) const;
     QRect decorationRect() const;
-public slots:
-    void discard(allowed_t = Allowed);
+    virtual Layer layer() const {
+        return m_layer;
+    }
 protected:
     virtual void debug(QDebug& stream) const;
     virtual bool shouldUnredirect() const;
@@ -86,6 +86,7 @@ private:
     QRect decoration_top;
     QRect decoration_bottom;
     int padding_left, padding_top, padding_right, padding_bottom;
+    Layer m_layer;
 };
 
 inline void Deleted::refWindow()
