@@ -60,9 +60,10 @@ BlurEffect::BlurEffect()
     } else {
         XDeleteProperty(display(), rootWindow(), net_wm_blur_region);
     }
-    connect(effects, SIGNAL(windowAdded(EffectWindow*)), this, SLOT(slotWindowAdded(EffectWindow*)));
-    connect(effects, SIGNAL(windowDeleted(EffectWindow*)), this, SLOT(slotWindowDeleted(EffectWindow*)));
-    connect(effects, SIGNAL(propertyNotify(EffectWindow*,long)), this, SLOT(slotPropertyNotify(EffectWindow*,long)));
+    connect(effects, SIGNAL(windowAdded(KWin::EffectWindow*)), this, SLOT(slotWindowAdded(KWin::EffectWindow*)));
+    connect(effects, SIGNAL(windowDeleted(KWin::EffectWindow*)), this, SLOT(slotWindowDeleted(KWin::EffectWindow*)));
+    connect(effects, SIGNAL(propertyNotify(KWin::EffectWindow*,long)), this, SLOT(slotPropertyNotify(KWin::EffectWindow*,long)));
+    connect(effects, SIGNAL(screenGeometryChanged(QSize)), this, SLOT(slotScreenGeometryChanged()));
 }
 
 BlurEffect::~BlurEffect()
@@ -76,6 +77,11 @@ BlurEffect::~BlurEffect()
     delete target;
 }
 
+void BlurEffect::slotScreenGeometryChanged()
+{
+    effects->reloadEffect(this);
+}
+
 void BlurEffect::reconfigure(ReconfigureFlags flags)
 {
     Q_UNUSED(flags)
@@ -133,9 +139,10 @@ void BlurEffect::slotPropertyNotify(EffectWindow *w, long atom)
 {
     if (w && atom == net_wm_blur_region) {
         updateBlurRegion(w);
-        if (windows.contains(w)) {
+        CacheEntry it = windows.find(w);
+        if (it != windows.end()) {
             const QRect screen(0, 0, displayWidth(), displayHeight());
-            windows[w].damagedRegion = expand(blurRegion(w).translated(w->pos())) & screen;
+            it->damagedRegion = expand(blurRegion(w).translated(w->pos())) & screen;
         }
     }
 }
@@ -144,7 +151,7 @@ bool BlurEffect::enabledByDefault()
 {
     GLPlatform *gl = GLPlatform::instance();
 
-    if (gl->isIntel())
+    if (gl->isIntel() && gl->chipClass() < SandyBridge)
         return false;
     if (gl->driver() == Driver_Catalyst) {
         // fglrx supports only ARB shaders and those tend to crash KWin (see Bug #270818 and #286795)
@@ -275,8 +282,9 @@ void BlurEffect::prePaintWindow(EffectWindow* w, WindowPrePaintData& data, int t
     }
 
     // in case this window has regions to be blurred
-    const QRegion blurArea = blurRegion(w).translated(w->pos());
-    const QRegion expandedBlur = expand(blurArea);
+    const QRect screen(0, 0, displayWidth(), displayHeight());
+    const QRegion blurArea = blurRegion(w).translated(w->pos()) & screen;
+    const QRegion expandedBlur = expand(blurArea) & screen;
 
     if (m_shouldCache) {
         // we are caching the horizontally blurred background texture
@@ -284,8 +292,12 @@ void BlurEffect::prePaintWindow(EffectWindow* w, WindowPrePaintData& data, int t
         // if a window underneath the blurred area is damaged we have to
         // update the cached texture
         QRegion damagedCache;
-        if (windows.contains(w) && !windows[w].dropCache) {
-            damagedCache = expand(expandedBlur & m_damagedArea) & expandedBlur;
+        CacheEntry it = windows.find(w);
+        if (it != windows.end() && !it->dropCache &&
+            it->windowPos == w->pos() &&
+            it->blurredBackground.size() == expandedBlur.boundingRect().size()) {
+            damagedCache = (expand(expandedBlur & m_damagedArea) |
+                            (it->damagedRegion & data.paint)) & expandedBlur;
         } else {
             damagedCache = expandedBlur;
         }
@@ -295,10 +307,14 @@ void BlurEffect::prePaintWindow(EffectWindow* w, WindowPrePaintData& data, int t
             // In order to be able to recalculate this area we have to make sure the
             // background area is painted before.
             data.paint |= expand(damagedArea);
-            if (windows.contains(w)) {
+            if (it != windows.end()) {
                 // In case we already have a texture cache mark the dirty regions invalid.
-                windows[w].damagedRegion |= damagedCache;
-                windows[w].dropCache = false;
+                it->damagedRegion &= expandedBlur;
+                it->damagedRegion |= damagedCache;
+                // The valid part of the cache can be considered as being opaque
+                // as long as we don't need to update a bordering part
+                data.clip |= blurArea - expand(it->damagedRegion);
+                it->dropCache = false;
             }
             // we keep track of the "damage propagation"
             m_damagedArea |= damagedArea;
@@ -426,9 +442,11 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
     // Set up the texture matrix to transform from screen coordinates
     // to texture coordinates.
 #ifndef KWIN_HAVE_OPENGLES
-    glMatrixMode(GL_TEXTURE);
+    if (!ShaderManager::instance()->isValid()) {
+        glMatrixMode(GL_TEXTURE);
+        pushMatrix();
+    }
 #endif
-    pushMatrix();
     QMatrix4x4 textureMatrix;
     textureMatrix.scale(1.0 / scratch.width(), -1.0 / scratch.height(), 1);
     textureMatrix.translate(-r.x(), -scratch.height() - r.y(), 0);
@@ -450,9 +468,6 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
 
     // Modulate the blurred texture with the window opacity if the window isn't opaque
     if (opacity < 1.0) {
-#ifndef KWIN_HAVE_OPENGLES
-        glPushAttrib(GL_COLOR_BUFFER_BIT);
-#endif
         glEnable(GL_BLEND);
         glBlendColor(0, 0, 0, opacity);
         glBlendFunc(GL_CONSTANT_ALPHA, GL_ONE_MINUS_CONSTANT_ALPHA);
@@ -468,16 +483,15 @@ void BlurEffect::doBlur(const QRegion& shape, const QRect& screen, const float o
 
     drawRegion(shape);
 
-    popMatrix();
 #ifndef KWIN_HAVE_OPENGLES
-    glMatrixMode(GL_MODELVIEW);
+    if (!ShaderManager::instance()->isValid()) {
+        popMatrix();
+        glMatrixMode(GL_MODELVIEW);
+    }
 #endif
 
     if (opacity < 1.0) {
         glDisable(GL_BLEND);
-#ifndef KWIN_HAVE_OPENGLES
-        glPopAttrib();
-#endif
     }
 
     tex.unbind();
@@ -493,47 +507,81 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
 
     // The background texture we get is only partially valid.
 
-    if (!windows.contains(w)) {
+    CacheEntry it = windows.find(w);
+    if (it == windows.end()) {
         BlurWindowInfo bwi;
         bwi.blurredBackground = GLTexture(r.width(),r.height());
         bwi.damagedRegion = expanded;
         bwi.dropCache = false;
-        windows[w] = bwi;
+        bwi.windowPos = w->pos();
+        it = windows.insert(w, bwi);
+    } else if (it->blurredBackground.size() != r.size()) {
+        it->blurredBackground = GLTexture(r.width(),r.height());
+        it->dropCache = false;
+        it->windowPos = w->pos();
+    } else if (it->windowPos != w->pos()) {
+        it->dropCache = false;
+        it->windowPos = w->pos();
     }
 
-    if (windows[w].blurredBackground.size() != r.size()) {
-        windows[w].blurredBackground = GLTexture(r.width(),r.height());
-        windows[w].damagedRegion = expanded;
-        windows[w].dropCache = false;
-    }
-
-    GLTexture targetTexture = windows[w].blurredBackground;
+    GLTexture targetTexture = it->blurredBackground;
     targetTexture.setFilter(GL_LINEAR);
     targetTexture.setWrapMode(GL_CLAMP_TO_EDGE);
     shader->bind();
     QMatrix4x4 textureMatrix;
     QMatrix4x4 modelViewProjectionMatrix;
 #ifndef KWIN_HAVE_OPENGLES
-    glMatrixMode(GL_MODELVIEW);
-    pushMatrix();
-    glLoadIdentity();
-    glMatrixMode(GL_TEXTURE);
-    pushMatrix();
-    glMatrixMode(GL_PROJECTION);
-    pushMatrix();
+    if (!ShaderManager::instance()->isValid()) {
+        glMatrixMode(GL_MODELVIEW);
+        pushMatrix();
+        glLoadIdentity();
+        glMatrixMode(GL_TEXTURE);
+        pushMatrix();
+        glMatrixMode(GL_PROJECTION);
+        pushMatrix();
+    }
 #endif
 
-    // We only update that part of the background texture that is visible and marked as dirty.
-    const QRegion updateBackground = windows[w].damagedRegion & region;
+    /**
+     * Which part of the background texture can be updated ?
+     *
+     * Well this is a rather difficult question. We kind of rely on the fact, that
+     * we need a bigger background region being painted before, more precisely if we want to
+     * blur region A we need the background region expand(A). This business logic is basically
+     * done in prePaintWindow:
+     *          data.paint |= expand(damagedArea);
+     *
+     * Now "data.paint" gets clipped and becomes what we receive as the "region" variable
+     * in this function. In theory there is now only one function that does this clipping
+     * and this is paintSimpleScreen. The clipping has the effect that "damagedRegion"
+     * is no longer a subset of "region" and we cannot fully validate the cache within one
+     * rendering pass. If we would now update the "damageRegion & region" part of the cache
+     * we would wrongly update the part of the cache that is next to the "region" border and
+     * which lies within "damagedRegion", just because we cannot assume that the framebuffer
+     * outside of "region" is valid. Therefore the maximal damaged region of the cache that can
+     * be repainted is given by:
+     *          validUpdate = damagedRegion - expand(damagedRegion - region);
+     *
+     * Now you may ask what is with the rest of "damagedRegion & region" that is not part
+     * of "validUpdate" but also might end up on the screen. Well under the assumption
+     * that only the occlusion culling can shrink "data.paint", we can control this by reducing
+     * the opaque area of every window by a margin of the blurring radius (c.f. prePaintWindow).
+     * This way we are sure that this area is overpainted by a higher opaque window.
+     *
+     * Apparently paintSimpleScreen is not the only function that can influence "region".
+     * In fact every effect's paintWindow that is called before Blur::paintWindow
+     * can do so (e.g. SlidingPopups). Hence we have to make the compromise that we update
+     * "damagedRegion & region" of the cache but only mark "validUpdate" as valid.
+     **/
+    const QRegion damagedRegion = it->damagedRegion;
+    const QRegion updateBackground = damagedRegion & region;
+    const QRegion validUpdate = damagedRegion - expand(damagedRegion - region);
 
-    if (!updateBackground.isEmpty()) {
+    if (!validUpdate.isEmpty()) {
         const QRect updateRect = (expand(updateBackground) & expanded).boundingRect();
-        // Create a scratch texture and copy the area in the back buffer that we're
-        // going to blur into it
-        GLTexture scratch(updateRect.width(), updateRect.height());
-        scratch.setFilter(GL_LINEAR);
-        scratch.setWrapMode(GL_CLAMP_TO_EDGE);
-        scratch.bind();
+        // First we have to copy the background from the frontbuffer
+        // into a scratch texture (in this case "tex").
+        tex.bind();
 
         glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, updateRect.x(), displayHeight() - updateRect.y() - updateRect.height(),
                             updateRect.width(), updateRect.height());
@@ -543,7 +591,7 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
         GLRenderTarget::pushRenderTarget(target);
 
         shader->setDirection(Qt::Horizontal);
-        shader->setPixelDistance(1.0 / updateRect.width());
+        shader->setPixelDistance(1.0 / tex.width());
 
         modelViewProjectionMatrix.ortho(0, r.width(), r.height(), 0 , 0, 65535);
         modelViewProjectionMatrix.translate(-r.x(), -r.y(), 0);
@@ -552,21 +600,23 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
 
         // Set up the texture matrix to transform from screen coordinates
         // to texture coordinates.
-        textureMatrix.scale(1.0 / scratch.width(), -1.0 / scratch.height(), 1);
-        textureMatrix.translate(-updateRect.x(), -scratch.height() - updateRect.y(), 0);
+        textureMatrix.scale(1.0 / tex.width(), -1.0 / tex.height(), 1);
+        textureMatrix.translate(-updateRect.x(), -updateRect.height() - updateRect.y(), 0);
 #ifndef KWIN_HAVE_OPENGLES
-        glMatrixMode(GL_TEXTURE);
-        loadMatrix(textureMatrix);
-        glMatrixMode(GL_PROJECTION);
+        if (!ShaderManager::instance()->isValid()) {
+            glMatrixMode(GL_TEXTURE);
+            loadMatrix(textureMatrix);
+            glMatrixMode(GL_PROJECTION);
+        }
 #endif
         shader->setTextureMatrix(textureMatrix);
 
         drawRegion(updateBackground & screen);
 
         GLRenderTarget::popRenderTarget();
-        scratch.unbind();
+        tex.unbind();
         // mark the updated region as valid
-        windows[w].damagedRegion -= updateBackground;
+        it->damagedRegion -= validUpdate;
     }
 
     // Now draw the horizontally blurred area back to the backbuffer, while
@@ -578,9 +628,6 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
 
     // Modulate the blurred texture with the window opacity if the window isn't opaque
     if (opacity < 1.0) {
-#ifndef KWIN_HAVE_OPENGLES
-        glPushAttrib(GL_COLOR_BUFFER_BIT);
-#endif
         glEnable(GL_BLEND);
         glBlendColor(0, 0, 0, opacity);
         glBlendFunc(GL_CONSTANT_ALPHA, GL_ONE_MINUS_CONSTANT_ALPHA);
@@ -597,27 +644,28 @@ void BlurEffect::doCachedBlur(EffectWindow *w, const QRegion& region, const floa
     textureMatrix.scale(1.0 / targetTexture.width(), -1.0 / targetTexture.height(), 1);
     textureMatrix.translate(-r.x(), -targetTexture.height() - r.y(), 0);
 #ifndef KWIN_HAVE_OPENGLES
-    glMatrixMode(GL_TEXTURE);
-    loadMatrix(textureMatrix);
-    glMatrixMode(GL_PROJECTION);
+    if (!ShaderManager::instance()->isValid()) {
+        glMatrixMode(GL_TEXTURE);
+        loadMatrix(textureMatrix);
+        glMatrixMode(GL_PROJECTION);
+    }
 #endif
     shader->setTextureMatrix(textureMatrix);
 
     drawRegion(blurredRegion & region);
 
 #ifndef KWIN_HAVE_OPENGLES
-    popMatrix();
-    glMatrixMode(GL_TEXTURE);
-    popMatrix();
-    glMatrixMode(GL_MODELVIEW);
-    popMatrix();
+    if (!ShaderManager::instance()->isValid()) {
+        popMatrix();
+        glMatrixMode(GL_TEXTURE);
+        popMatrix();
+        glMatrixMode(GL_MODELVIEW);
+        popMatrix();
+    }
 #endif
 
     if (opacity < 1.0) {
         glDisable(GL_BLEND);
-#ifndef KWIN_HAVE_OPENGLES
-        glPopAttrib();
-#endif
     }
 
     targetTexture.unbind();
