@@ -42,22 +42,24 @@ public:
     virtual void postPaintScreen();
     virtual void reconfigure(ReconfigureFlags);
     virtual bool isActive() const;
-    static bool supported();
 private slots:
     void toggle();
     void slotMouseChanged(const QPoint& pos, const QPoint& old,
                               Qt::MouseButtons buttons, Qt::MouseButtons oldbuttons,
                               Qt::KeyboardModifiers modifiers, Qt::KeyboardModifiers oldmodifiers);
 private:
-    QRect starRect(int num) const;
+    bool init();
     void loadTexture();
-    bool active, mousePolling;
-    int angle;
-    GLTexture* texture;
-    QSize textureSize;
-    KActionCollection* actionCollection;
-    KAction* action;
-    Qt::KeyboardModifiers modifier;
+    QRect m_lastRect[2];
+    bool m_active, m_mousePolling;
+    float m_angle;
+    float m_angleBase;
+    GLTexture* m_texture[2];
+#ifdef KWIN_HAVE_XRENDER_COMPOSITING
+    QPixmap *m_pixmap[2];
+#endif
+    KAction* m_action;
+    Qt::KeyboardModifiers m_modifiers;
 };
 
 } // namespace
