@@ -115,7 +115,7 @@ void Workspace::updateStackingOrder(bool propagate_new_clients)
             blocked_propagating_new_clients = true;
         return;
     }
-    ClientList new_stacking_order = constrainedStackingOrder();
+    ToplevelList new_stacking_order = constrainedStackingOrder();
     bool changed = (new_stacking_order != stacking_order || force_restacking);
     force_restacking = false;
     stacking_order = new_stacking_order;
@@ -163,8 +163,8 @@ void Workspace::propagateClients(bool propagate_new_clients)
     }
 #endif
     for (int i = stacking_order.size() - 1; i >= 0; i--) {
-        Client *client = stacking_order.at(i);
-        if (client->hiddenPreview()) {
+        Client *client = qobject_cast<Client*>(stacking_order.at(i));
+        if (!client || client->hiddenPreview()) {
             continue;
         }
 
@@ -179,9 +179,10 @@ void Workspace::propagateClients(bool propagate_new_clients)
     // (as far as pure X stacking order is concerned), in order to avoid having
     // these windows that should be unmapped to interfere with other windows
     for (int i = stacking_order.size() - 1; i >= 0; i--) {
-        if (!stacking_order.at(i)->hiddenPreview())
+        Client *client = qobject_cast<Client*>(stacking_order.at(i));
+        if (!client || !client->hiddenPreview())
             continue;
-        newWindowStack << (Window*)stacking_order.at(i)->frameId();
+        newWindowStack << (Window*)client->frameId();
     }
     // TODO isn't it too inefficient to restack always all clients?
     // TODO don't restack not visible windows?
@@ -202,7 +203,7 @@ void Workspace::propagateClients(bool propagate_new_clients)
 
     cl = new Window[ stacking_order.count()];
     pos = 0;
-    for (ClientList::ConstIterator it = stacking_order.constBegin(); it != stacking_order.constEnd(); ++it)
+    for (ToplevelList::ConstIterator it = stacking_order.constBegin(); it != stacking_order.constEnd(); ++it)
         cl[pos++] = (*it)->window();
     rootInfo->setClientListStacking(cl, pos);
     delete [] cl;
@@ -221,7 +222,7 @@ void Workspace::propagateClients(bool propagate_new_clients)
 Client* Workspace::topClientOnDesktop(int desktop, int screen, bool unconstrained, bool only_normal) const
 {
 // TODO    Q_ASSERT( block_stacking_updates == 0 );
-    ClientList list;
+    ToplevelList list;
     if (!unconstrained)
         list = stacking_order;
     else
@@ -229,13 +230,17 @@ Client* Workspace::topClientOnDesktop(int desktop, int screen, bool unconstraine
     for (int i = list.size() - 1;
             i >= 0;
             --i) {
-        if (list.at(i)->isOnDesktop(desktop) && list.at(i)->isShown(false) && list.at(i)->isOnCurrentActivity()) {
-            if (screen != -1 && list.at(i)->screen() != screen)
+        Client *c = qobject_cast<Client*>(list.at(i));
+        if (!c) {
+            continue;
+        }
+        if (c->isOnDesktop(desktop) && c->isShown(false) && c->isOnCurrentActivity()) {
+            if (screen != -1 && c->screen() != screen)
                 continue;
             if (!only_normal)
-                return list.at(i);
-            if (list.at(i)->wantsTabFocus() && !list.at(i)->isSpecialWindow())
-                return list.at(i);
+                return c;
+            if (c->wantsTabFocus() && !c->isSpecialWindow())
+                return c;
         }
     }
     return 0;
@@ -246,16 +251,18 @@ Client* Workspace::findDesktop(bool topmost, int desktop) const
 // TODO    Q_ASSERT( block_stacking_updates == 0 );
     if (topmost) {
         for (int i = stacking_order.size() - 1; i >= 0; i--) {
-            if (stacking_order.at(i)->isOnDesktop(desktop) && stacking_order.at(i)->isDesktop()
-                    && stacking_order.at(i)->isShown(true))
-                return stacking_order.at(i);
-        }
-    } else { // bottom-most
-        foreach (Client * c, stacking_order) {
-            if (c->isOnDesktop(desktop) && c->isDesktop()
+            Client *c = qobject_cast<Client*>(stacking_order.at(i));
+            if (c && c->isOnDesktop(desktop) && c->isDesktop()
                     && c->isShown(true))
                 return c;
         }
+    } else { // bottom-most
+        foreach (Toplevel * c, stacking_order) {
+            Client *client = qobject_cast<Client*>(c);
+            if (client && c->isOnDesktop(desktop) && c->isDesktop()
+                    && client->isShown(true))
+                return client;
+        }
     }
     return NULL;
 }
@@ -270,7 +277,7 @@ void Workspace::raiseOrLowerClient(Client *c)
         topmost = most_recently_raised;
     else
         topmost = topClientOnDesktop(c->isOnAllDesktops() ? currentDesktop() : c->desktop(),
-                                     options->separateScreenFocus ? c->screen() : -1);
+                                     options->isSeparateScreenFocus() ? c->screen() : -1);
 
     if (c == topmost)
         lowerClient(c);
@@ -317,14 +324,19 @@ void Workspace::lowerClientWithinApplication(Client* c)
     unconstrained_stacking_order.removeAll(c);
     bool lowered = false;
     // first try to put it below the bottom-most window of the application
-    for (ClientList::Iterator it = unconstrained_stacking_order.begin();
+    for (ToplevelList::Iterator it = unconstrained_stacking_order.begin();
             it != unconstrained_stacking_order.end();
-            ++it)
-        if (Client::belongToSameApplication(*it, c)) {
+            ++it) {
+        Client *client = qobject_cast<Client*>(*it);
+        if (!client) {
+            continue;
+        }
+        if (Client::belongToSameApplication(client, c)) {
             unconstrained_stacking_order.insert(it, c);
             lowered = true;
             break;
         }
+    }
     if (!lowered)
         unconstrained_stacking_order.prepend(c);
     // ignore mainwindows
@@ -369,7 +381,10 @@ void Workspace::raiseClientWithinApplication(Client* c)
 
     // first try to put it above the top-most window of the application
     for (int i = unconstrained_stacking_order.size() - 1; i > -1 ; --i) {
-        Client *other = unconstrained_stacking_order.at(i);
+        Client *other = qobject_cast<Client*>(unconstrained_stacking_order.at(i));
+        if (!other) {
+            continue;
+        }
         if (other == c)     // don't lower it just because it asked to be raised
             return;
         if (Client::belongToSameApplication(other, c)) {
@@ -408,9 +423,9 @@ void Workspace::restack(Client* c, Client* under)
     assert(unconstrained_stacking_order.contains(under));
     if (!Client::belongToSameApplication(under, c)) {
          // put in the stacking order below _all_ windows belonging to the active application
-        Client *other = 0;
-        for (int i = 0; i < unconstrained_stacking_order.size(); ++i) { // TODO ignore topmenus?
-            if (Client::belongToSameApplication(under, (other = unconstrained_stacking_order.at(i)))) {
+        for (int i = 0; i < unconstrained_stacking_order.size(); ++i) {
+            Client *other = qobject_cast<Client*>(unconstrained_stacking_order.at(i));
+            if (other && Client::belongToSameApplication(under, other)) {
                 under = (c == other) ? 0 : other;
                 break;
             }
@@ -474,11 +489,14 @@ void Workspace::restoreSessionStackingOrder(Client* c)
         return;
     StackingUpdatesBlocker blocker(this);
     unconstrained_stacking_order.removeAll(c);
-    ClientList::Iterator best_pos = unconstrained_stacking_order.end();
-    for (ClientList::Iterator it = unconstrained_stacking_order.begin();  // from bottom
+    for (ToplevelList::Iterator it = unconstrained_stacking_order.begin();  // from bottom
             it != unconstrained_stacking_order.end();
             ++it) {
-        if ((*it)->sessionStackingOrder() > c->sessionStackingOrder()) {
+        Client *current = qobject_cast<Client*>(*it);
+        if (!current) {
+            continue;
+        }
+        if (current->sessionStackingOrder() > c->sessionStackingOrder()) {
             unconstrained_stacking_order.insert(it, c);
             return;
         }
@@ -503,9 +521,9 @@ void Workspace::circulateDesktopApplications()
 /*!
   Returns a stacking order based upon \a list that fulfills certain contained.
  */
-ClientList Workspace::constrainedStackingOrder()
+ToplevelList Workspace::constrainedStackingOrder()
 {
-    ClientList layer[ NumLayers ];
+    ToplevelList layer[ NumLayers ];
 
 #if 0
     kDebug(1212) << "stacking1:";
@@ -516,12 +534,13 @@ ClientList Workspace::constrainedStackingOrder()
 #endif
     // build the order from layers
     QVector< QMap<Group*, Layer> > minimum_layer(numScreens());
-    for (ClientList::ConstIterator it = unconstrained_stacking_order.constBegin(),
+    for (ToplevelList::ConstIterator it = unconstrained_stacking_order.constBegin(),
                                   end = unconstrained_stacking_order.constEnd(); it != end; ++it) {
         Layer l = (*it)->layer();
 
         const int screen = (*it)->screen();
-        QMap< Group*, Layer >::iterator mLayer = minimum_layer[screen].find((*it)->group());
+        Client *c = qobject_cast<Client*>(*it);
+        QMap< Group*, Layer >::iterator mLayer = minimum_layer[screen].find(c ? c->group() : NULL);
         if (mLayer != minimum_layer[screen].end()) {
             // If a window is raised above some other window in the same window group
             // which is in the ActiveLayer (i.e. it's fulscreened), make sure it stays
@@ -529,12 +548,12 @@ ClientList Workspace::constrainedStackingOrder()
             if (*mLayer == ActiveLayer && (l == NormalLayer || l == AboveLayer))
                 l = ActiveLayer;
             *mLayer = l;
-        } else {
-            minimum_layer[screen].insertMulti((*it)->group(), l);
+        } else if (c) {
+            minimum_layer[screen].insertMulti(c->group(), l);
         }
         layer[ l ].append(*it);
     }
-    ClientList stacking;
+    ToplevelList stacking;
     for (Layer lay = FirstLayer;
             lay < NumLayers;
             ++lay)
@@ -551,13 +570,14 @@ ClientList Workspace::constrainedStackingOrder()
     for (int i = stacking.size() - 1;
             i >= 0;
        ) {
-        if (!stacking[ i ]->isTransient()) {
+        Client *current = qobject_cast<Client*>(stacking[i]);
+        if (!current || !current->isTransient()) {
             --i;
             continue;
         }
         int i2 = -1;
-        if (stacking[ i ]->groupTransient()) {
-            if (stacking[ i ]->group()->members().count() > 0) {
+        if (current->groupTransient()) {
+            if (current->group()->members().count() > 0) {
                 // find topmost client this one is transient for
                 for (i2 = stacking.size() - 1;
                         i2 >= 0;
@@ -566,8 +586,12 @@ ClientList Workspace::constrainedStackingOrder()
                         i2 = -1; // don't reorder, already the topmost in the group
                         break;
                     }
-                    if (stacking[ i2 ]->hasTransient(stacking[ i ], true)
-                            && keepTransientAbove(stacking[ i2 ], stacking[ i ]))
+                    Client *c2 = qobject_cast<Client*>(stacking[ i2 ]);
+                    if (!c2) {
+                        continue;
+                    }
+                    if (c2->hasTransient(current, true)
+                            && keepTransientAbove(c2, current))
                         break;
                 }
             } // else i2 remains pointing at -1
@@ -575,12 +599,16 @@ ClientList Workspace::constrainedStackingOrder()
             for (i2 = stacking.size() - 1;
                     i2 >= 0;
                     --i2) {
-                if (stacking[ i2 ] == stacking[ i ]) {
+                Client *c2 = qobject_cast<Client*>(stacking[ i2 ]);
+                if (!c2) {
+                    continue;
+                }
+                if (c2 == current) {
                     i2 = -1; // don't reorder, already on top of its mainwindow
                     break;
                 }
-                if (stacking[ i2 ] == stacking[ i ]->transientFor()
-                        && keepTransientAbove(stacking[ i2 ], stacking[ i ]))
+                if (c2 == current->transientFor()
+                        && keepTransientAbove(c2, current))
                     break;
             }
         }
@@ -588,7 +616,6 @@ ClientList Workspace::constrainedStackingOrder()
             --i;
             continue;
         }
-        Client* current = stacking[ i ];
         stacking.removeAt(i);
         --i; // move onto the next item (for next for () iteration)
         --i2; // adjust index of the mainwindow after the remove above
@@ -630,11 +657,16 @@ ClientList Workspace::ensureStackingOrder(const ClientList& list) const
         return list;
     // TODO is this worth optimizing?
     ClientList result = list;
-    for (ClientList::ConstIterator it = stacking_order.constBegin();
+    for (ToplevelList::ConstIterator it = stacking_order.constBegin();
             it != stacking_order.constEnd();
-            ++it)
-        if (result.removeAll(*it) != 0)
-            result.append(*it);
+            ++it) {
+        Client *c = qobject_cast<Client*>(*it);
+        if (!c) {
+            continue;
+        }
+        if (result.removeAll(c) != 0)
+            result.append(c);
+    }
     return result;
 }
 
@@ -670,7 +702,7 @@ ToplevelList Workspace::xStackingOrder() const
     unsigned int count = 0;
     XQueryTree(display(), rootWindow(), &dummy, &dummy, &windows, &count);
     // use our own stacking order, not the X one, as they may differ
-    foreach (Client * c, stacking_order)
+    foreach (Toplevel * c, stacking_order)
     x_stacking.append(c);
     for (unsigned int i = 0;
             i < count;
@@ -678,8 +710,6 @@ ToplevelList Workspace::xStackingOrder() const
         if (Unmanaged* c = findUnmanaged(WindowMatchPredicate(windows[ i ])))
             x_stacking.append(c);
     }
-    foreach (Deleted * c, deleted)
-    x_stacking.append(c);
     if (windows != NULL)
         XFree(windows);
     const_cast< Workspace* >(this)->checkUnredirect();
@@ -699,7 +729,7 @@ void Client::restackWindow(Window above, int detail, NET::RequestSource src, Tim
             workspace()->raiseOrLowerClient(this);
             return;
         }
-        ClientList::const_iterator  it = workspace()->stackingOrder().constBegin(),
+        ToplevelList::const_iterator  it = workspace()->stackingOrder().constBegin(),
                                     end = workspace()->stackingOrder().constEnd();
         while (it != end) {
             if (*it == this) {
@@ -729,7 +759,7 @@ void Client::restackWindow(Window above, int detail, NET::RequestSource src, Tim
         other = workspace()->findClient(WindowMatchPredicate(above));
 
     if (other && detail == Above) {
-        ClientList::const_iterator  it = workspace()->stackingOrder().constEnd(),
+        ToplevelList::const_iterator  it = workspace()->stackingOrder().constEnd(),
                                     begin = workspace()->stackingOrder().constBegin();
         while (--it != begin) {
             if (*it == this)
@@ -740,8 +770,9 @@ void Client::restackWindow(Window above, int detail, NET::RequestSource src, Tim
                 src = NET::FromTool; // force
                 break;
             }
+            Client *c = qobject_cast<Client*>(*it);
 
-            if (!(  (*it)->isNormalWindow() && (*it)->isShown(true) &&
+            if (!c || !(  (*it)->isNormalWindow() && c->isShown(true) &&
                     (*it)->isOnCurrentDesktop() && (*it)->isOnCurrentActivity() && (*it)->isOnScreen(screen()) ))
                 continue; // irrelevant clients
 
@@ -750,7 +781,7 @@ void Client::restackWindow(Window above, int detail, NET::RequestSource src, Tim
         }
 
         if (it != begin && (*(it - 1) == other))
-            other = const_cast<Client*>(*it);
+            other = qobject_cast<Client*>(*it);
         else
             other = 0;
     }
@@ -782,11 +813,12 @@ void Client::setKeepAbove(bool b)
     if (decoration != NULL)
         decoration->emitKeepAboveChanged(keepAbove());
     workspace()->updateClientLayer(this);
-    updateWindowRules();
+    updateWindowRules(Rules::Above);
 
     // Update states of all other windows in this group
-    if (clientGroup())
-        clientGroup()->updateStates(this);
+    if (tabGroup())
+        tabGroup()->updateStates(this, TabGroup::Layer);
+    emit keepAboveChanged();
 }
 
 void Client::setKeepBelow(bool b)
@@ -805,11 +837,12 @@ void Client::setKeepBelow(bool b)
     if (decoration != NULL)
         decoration->emitKeepBelowChanged(keepBelow());
     workspace()->updateClientLayer(this);
-    updateWindowRules();
+    updateWindowRules(Rules::Below);
 
     // Update states of all other windows in this group
-    if (clientGroup())
-        clientGroup()->updateStates(this);
+    if (tabGroup())
+        tabGroup()->updateStates(this, TabGroup::Layer);
+    emit keepBelowChanged();
 }
 
 Layer Client::layer() const
@@ -825,15 +858,18 @@ Layer Client::belongsToLayer() const
         return DesktopLayer;
     if (isSplash())          // no damn annoying splashscreens
         return NormalLayer; // getting in the way of everything else
-    if (isDock() && keepBelow())
+    if (isDock()) {
         // slight hack for the 'allow window to cover panel' Kicker setting
         // don't move keepbelow docks below normal window, but only to the same
         // layer, so that both may be raised to cover the other
-        return NormalLayer;
+        if (keepBelow())
+            return NormalLayer;
+        if (keepAbove()) // slight hack for the autohiding panels
+            return AboveLayer;
+        return DockLayer;
+    }
     if (keepBelow())
         return BelowLayer;
-    if (isDock() && !keepBelow())
-        return DockLayer;
     if (isActiveFullScreen())
         return ActiveLayer;
     if (keepAbove())
