@@ -105,8 +105,7 @@ void initGL()
 #ifdef KWIN_HAVE_OPENGLES
     legacyGl = false;
 #else
-    KSharedConfig::Ptr kwinconfig = KSharedConfig::openConfig("kwinrc", KConfig::NoGlobals);
-    KConfigGroup config(kwinconfig, "Compositing");
+    KConfigGroup config(KGlobal::config(), "Compositing");
     legacyGl = config.readEntry<bool>("GLLegacy", false);
     glVersion = MAKE_GL_VERSION(glversioninfo[0].toInt(), glversioninfo[1].toInt(), glversioninfo[2].toInt());
 #endif
@@ -225,6 +224,9 @@ void loadMatrix(const QMatrix4x4 &matrix)
 #ifdef KWIN_HAVE_OPENGLES
     Q_UNUSED(matrix)
 #else
+    if (ShaderManager::instance()->isValid()) {
+        return;
+    }
     GLfloat m[16];
     const qreal *data = matrix.constData();
     for (int i = 0; i < 4; ++i) {
@@ -689,6 +691,20 @@ GLShader *ShaderManager::pushShader(ShaderType type, bool reset)
     return shader;
 }
 
+void ShaderManager::resetAllShaders()
+{
+    if (!m_inited || !m_valid) {
+        return;
+    }
+    pushShader(SimpleShader, true);
+    pushShader(GenericShader, true);
+    pushShader(ColorShader, true);
+    popShader();
+    popShader();
+    popShader();
+}
+
+
 void ShaderManager::pushShader(GLShader *shader)
 {
     // only bind shader if it is not already bound
@@ -1061,7 +1077,11 @@ void GLRenderTarget::blitFromFramebuffer(const QRect &source, const QRect &desti
     if (!GLRenderTarget::blitSupported()) {
         return;
     }
-#ifndef KWIN_HAVE_OPENGLES
+#ifdef KWIN_HAVE_OPENGLES
+    Q_UNUSED(source)
+    Q_UNUSED(destination)
+    Q_UNUSED(filter)
+#else
     GLRenderTarget::pushRenderTarget(this);
     glBindFramebuffer(GL_DRAW_FRAMEBUFFER, mFramebuffer);
     glBindFramebuffer(GL_READ_FRAMEBUFFER, 0);
@@ -1125,21 +1145,22 @@ public:
     QColor color;
 
     //! VBO is not supported
-    void legacyPainting(QRegion region, GLenum primitiveMode);
+    void legacyPainting(QRegion region, GLenum primitiveMode, bool hardwareClipping);
     //! VBO and shaders are both supported
-    void corePainting(const QRegion& region, GLenum primitiveMode);
+    void corePainting(const QRegion& region, GLenum primitiveMode, bool hardwareClipping);
     //! VBO is supported, but shaders are not supported
-    void fallbackPainting(const QRegion& region, GLenum primitiveMode);
+    void fallbackPainting(const QRegion& region, GLenum primitiveMode, bool hardwareClipping);
 };
 bool GLVertexBufferPrivate::supported = false;
 GLVertexBuffer *GLVertexBufferPrivate::streamingBuffer = NULL;
 
-void GLVertexBufferPrivate::legacyPainting(QRegion region, GLenum primitiveMode)
+void GLVertexBufferPrivate::legacyPainting(QRegion region, GLenum primitiveMode, bool hardwareClipping)
 {
 #ifdef KWIN_HAVE_OPENGLES
-    Q_UNUSED(primitiveMode)
-#else
     Q_UNUSED(region)
+    Q_UNUSED(primitiveMode)
+    Q_UNUSED(hardwareClipping)
+#else
     // Enable arrays
     glEnableClientState(GL_VERTEX_ARRAY);
     glVertexPointer(dimension, GL_FLOAT, 0, legacyVertices.constData());
@@ -1152,7 +1173,14 @@ void GLVertexBufferPrivate::legacyPainting(QRegion region, GLenum primitiveMode)
         glColor4f(color.redF(), color.greenF(), color.blueF(), color.alphaF());
     }
 
-    glDrawArrays(primitiveMode, 0, numberVertices);
+    if (!hardwareClipping) {
+        glDrawArrays(primitiveMode, 0, numberVertices);
+    } else {
+        foreach (const QRect& r, region.rects()) {
+            glScissor(r.x(), displayHeight() - r.y() - r.height(), r.width(), r.height());
+            glDrawArrays(primitiveMode, 0, numberVertices);
+        }
+    }
 
     glDisableClientState(GL_VERTEX_ARRAY);
     if (!legacyTexCoords.isEmpty()) {
@@ -1161,9 +1189,8 @@ void GLVertexBufferPrivate::legacyPainting(QRegion region, GLenum primitiveMode)
 #endif
 }
 
-void GLVertexBufferPrivate::corePainting(const QRegion& region, GLenum primitiveMode)
+void GLVertexBufferPrivate::corePainting(const QRegion& region, GLenum primitiveMode, bool hardwareClipping)
 {
-    Q_UNUSED(region)
     GLShader *shader = ShaderManager::instance()->getBoundShader();
     GLint vertexAttrib = shader->attributeLocation("vertex");
     GLint texAttrib = shader->attributeLocation("texCoord");
@@ -1185,7 +1212,14 @@ void GLVertexBufferPrivate::corePainting(const QRegion& region, GLenum primitive
         glVertexAttribPointer(texAttrib, 2, GL_FLOAT, GL_FALSE, 0, 0);
     }
 
-    glDrawArrays(primitiveMode, 0, numberVertices);
+    if (!hardwareClipping) {
+        glDrawArrays(primitiveMode, 0, numberVertices);
+    } else {
+        foreach (const QRect& r, region.rects()) {
+            glScissor(r.x(), displayHeight() - r.y() - r.height(), r.width(), r.height());
+            glDrawArrays(primitiveMode, 0, numberVertices);
+        }
+    }
 
     glBindBuffer(GL_ARRAY_BUFFER, 0);
 
@@ -1195,12 +1229,13 @@ void GLVertexBufferPrivate::corePainting(const QRegion& region, GLenum primitive
     glDisableVertexAttribArray(vertexAttrib);
 }
 
-void GLVertexBufferPrivate::fallbackPainting(const QRegion& region, GLenum primitiveMode)
+void GLVertexBufferPrivate::fallbackPainting(const QRegion& region, GLenum primitiveMode, bool hardwareClipping)
 {
 #ifdef KWIN_HAVE_OPENGLES
-    Q_UNUSED(primitiveMode)
-#else
     Q_UNUSED(region)
+    Q_UNUSED(primitiveMode)
+    Q_UNUSED(hardwareClipping)
+#else
     glEnableClientState(GL_VERTEX_ARRAY);
     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
     glBindBuffer(GL_ARRAY_BUFFER, buffers[ 0 ]);
@@ -1214,7 +1249,14 @@ void GLVertexBufferPrivate::fallbackPainting(const QRegion& region, GLenum primi
     }
 
     // Clip using scissoring
-    glDrawArrays(primitiveMode, 0, numberVertices);
+    if (!hardwareClipping) {
+        glDrawArrays(primitiveMode, 0, numberVertices);
+    } else {
+        foreach (const QRect& r, region.rects()) {
+            glScissor(r.x(), displayHeight() - r.y() - r.height(), r.width(), r.height());
+            glDrawArrays(primitiveMode, 0, numberVertices);
+        }
+    }
 
     glBindBuffer(GL_ARRAY_BUFFER, 0);
 
@@ -1286,17 +1328,17 @@ void GLVertexBuffer::setData(int numberVertices, int dim, const float* vertices,
 
 void GLVertexBuffer::render(GLenum primitiveMode)
 {
-    render(infiniteRegion(), primitiveMode);
+    render(infiniteRegion(), primitiveMode, false);
 }
 
-void GLVertexBuffer::render(const QRegion& region, GLenum primitiveMode)
+void GLVertexBuffer::render(const QRegion& region, GLenum primitiveMode, bool hardwareClipping)
 {
     if (!GLVertexBufferPrivate::supported) {
-        d->legacyPainting(region, primitiveMode);
+        d->legacyPainting(region, primitiveMode, hardwareClipping);
     } else if (ShaderManager::instance()->isShaderBound()) {
-        d->corePainting(region, primitiveMode);
+        d->corePainting(region, primitiveMode, hardwareClipping);
     } else {
-        d->fallbackPainting(region, primitiveMode);
+        d->fallbackPainting(region, primitiveMode, hardwareClipping);
     }
 }
 
