@@ -73,6 +73,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <X11/extensions/shape.h>
 
+#include <QGraphicsScene>
+#include <QGraphicsView>
+
 #include "client.h"
 #include "deleted.h"
 #include "effects.h"
@@ -80,7 +83,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "overlaywindow.h"
 #include "shadow.h"
 
-#include <kephal/screens.h>
 #include "thumbnailitem.h"
 
 namespace KWin
@@ -94,7 +96,6 @@ Scene* scene = 0;
 
 Scene::Scene(Workspace* ws)
     : QObject(ws)
-    , lastRenderTime(0)
     , wspace(ws)
     , has_waitSync(false)
     , lanczos_filter(new LanczosFilter())
@@ -122,20 +123,9 @@ void Scene::paintScreen(int* mask, QRegion* region)
     pdata.mask = *mask;
     pdata.paint = *region;
 
-    // region only includes all workspace-specific repaints but some effect (e.g. blur)
-    // rely on the full damaged area
-    QRegion dirtyArea;
-    foreach (Window * w, stacking_order) { // bottom to top
-        Toplevel* topw = w->window();
-        dirtyArea |= topw->repaints().translated(topw->pos());
-        dirtyArea |= topw->decorationPendingRegion();
-    }
-    pdata.paint |= dirtyArea;
-
     effects->prePaintScreen(pdata, time_diff);
     *mask = pdata.mask;
-    // Subtract the dirty region and let finalPaintScreen decide which areas have to be drawn
-    *region |= pdata.paint - dirtyArea;
+    *region = pdata.paint;
 
     if (*mask & (PAINT_SCREEN_TRANSFORMED | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS)) {
         // Region painting is not possible with transformations,
@@ -150,12 +140,14 @@ void Scene::paintScreen(int* mask, QRegion* region)
         *region = QRegion(0, 0, displayWidth(), displayHeight());
     }
     painted_region = *region;
-    if (*mask & PAINT_SCREEN_BACKGROUND_FIRST)
+    if (*mask & PAINT_SCREEN_BACKGROUND_FIRST) {
         paintBackground(*region);
+    }
     ScreenPaintData data;
     effects->paintScreen(*mask, *region, data);
-    foreach (Window * w, stacking_order)
-    effects->postPaintWindow(effectWindow(w));
+    foreach (Window * w, stacking_order) {
+        effects->postPaintWindow(effectWindow(w));
+    }
     effects->postPaintScreen();
     *region |= painted_region;
     // make sure not to go outside of the screen area
@@ -175,7 +167,11 @@ void Scene::updateTimeDiff()
         time_diff = 1;
         last_time.start();
     } else
-        time_diff = last_time.restart();
+
+    // the extra wspace->nextFrameDelay() basically means that we lie to the effect about the passed
+    // time - as a result the (animated) effect will run up to a frame shorter but in return stick
+    // closer to the runtime from the trigger
+    time_diff = last_time.restart() + wspace->nextFrameDelay();
 
     if (time_diff < 0)   // check time rollback
         time_diff = 1;
@@ -191,8 +187,7 @@ void Scene::idle()
 // the function that'll be eventually called by paintScreen() above
 void Scene::finalPaintScreen(int mask, QRegion region, ScreenPaintData& data)
 {
-    if (mask & (PAINT_SCREEN_TRANSFORMED | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS
-               | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_WITHOUT_FULL_REPAINTS))
+    if (mask & (PAINT_SCREEN_TRANSFORMED | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS))
         paintGenericScreen(mask, data);
     else
         paintSimpleScreen(mask, region);
@@ -202,18 +197,17 @@ void Scene::finalPaintScreen(int mask, QRegion region, ScreenPaintData& data)
 // It simply paints bottom-to-top.
 void Scene::paintGenericScreen(int orig_mask, ScreenPaintData)
 {
-    if (!(orig_mask & PAINT_SCREEN_BACKGROUND_FIRST))
+    if (!(orig_mask & PAINT_SCREEN_BACKGROUND_FIRST)) {
         paintBackground(infiniteRegion());
+    }
     QList< Phase2Data > phase2;
     foreach (Window * w, stacking_order) { // bottom to top
         Toplevel* topw = w->window();
-        painted_region |= topw->repaints().translated(topw->pos());
-        painted_region |= topw->decorationPendingRegion();
 
         // Reset the repaint_region.
         // This has to be done here because many effects schedule a repaint for
         // the next frame within Effects::prePaintWindow.
-        topw->resetRepaints(topw->visibleRect().translated(-topw->pos()));
+        topw->resetRepaints();
 
         WindowPrePaintData data;
         data.mask = orig_mask | (w->isOpaque() ? PAINT_WINDOW_OPAQUE : PAINT_WINDOW_TRANSLUCENT);
@@ -224,20 +218,22 @@ void Scene::paintGenericScreen(int orig_mask, ScreenPaintData)
         // preparation step
         effects->prePaintWindow(effectWindow(w), data, time_diff);
 #ifndef NDEBUG
-        foreach (const WindowQuad & q, data.quads)
-        if (q.isTransformed())
+        if (data.quads.isTransformed()) {
             kFatal(1212) << "Pre-paint calls are not allowed to transform quads!" ;
+        }
 #endif
-        if (!w->isPaintingEnabled())
+        if (!w->isPaintingEnabled()) {
             continue;
+        }
         phase2.append(Phase2Data(w, infiniteRegion(), data.clip, data.mask, data.quads));
         // transformations require window pixmap
         w->suspendUnredirect(data.mask
                              & (PAINT_WINDOW_TRANSLUCENT | PAINT_SCREEN_TRANSFORMED | PAINT_WINDOW_TRANSFORMED));
     }
 
-    foreach (const Phase2Data & d, phase2)
-    paintWindow(d.window, d.mask, d.region, d.quads);
+    foreach (const Phase2Data & d, phase2) {
+        paintWindow(d.window, d.mask, d.region, d.quads);
+    }
 }
 
 // The optimized case without any transformations at all.
@@ -245,13 +241,9 @@ void Scene::paintGenericScreen(int orig_mask, ScreenPaintData)
 // to reduce painting and improve performance.
 void Scene::paintSimpleScreen(int orig_mask, QRegion region)
 {
-    // TODO PAINT_WINDOW_* flags don't belong here, that's why it's in the assert,
-    // perhaps the two enums should be separated
-    assert((orig_mask & (PAINT_WINDOW_TRANSFORMED | PAINT_SCREEN_TRANSFORMED
-                         | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS
-                         | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_WITHOUT_FULL_REPAINTS
-                         | PAINT_WINDOW_TRANSLUCENT | PAINT_WINDOW_OPAQUE)) == 0);
-    QHash< Window*, Phase2Data > phase2data;
+    assert((orig_mask & (PAINT_SCREEN_TRANSFORMED
+                         | PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS)) == 0);
+    QList< QPair< Window*, Phase2Data > > phase2data;
 
     QRegion dirtyArea = region;
     for (int i = 0;  // do prePaintWindow bottom to top
@@ -263,13 +255,14 @@ void Scene::paintSimpleScreen(int orig_mask, QRegion region)
         data.mask = orig_mask | (w->isOpaque() ? PAINT_WINDOW_OPAQUE : PAINT_WINDOW_TRANSLUCENT);
         w->resetPaintingEnabled();
         data.paint = region;
-        data.paint |= topw->repaints().translated(topw->pos());
+        data.paint |= topw->repaints();
         data.paint |= topw->decorationPendingRegion();
 
         // Reset the repaint_region.
         // This has to be done here because many effects schedule a repaint for
         // the next frame within Effects::prePaintWindow.
-        topw->resetRepaints(topw->visibleRect().translated(-topw->pos()));
+        topw->resetRepaints();
+
         // Clip out the decoration for opaque windows; the decoration is drawn in the second pass
         if (w->isOpaque()) {
             // the window is fully opaque
@@ -284,11 +277,9 @@ void Scene::paintSimpleScreen(int orig_mask, QRegion region)
         // preparation step
         effects->prePaintWindow(effectWindow(w), data, time_diff);
 #ifndef NDEBUG
-        foreach (const WindowQuad & q, data.quads)
-        if (q.isTransformed())
+        if (data.quads.isTransformed()) {
             kFatal(1212) << "Pre-paint calls are not allowed to transform quads!" ;
-        if (data.mask & PAINT_WINDOW_TRANSFORMED)
-            kFatal(1212) << "PAINT_WINDOW_TRANSFORMED without PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS!";
+        }
 #endif
         if (!w->isPaintingEnabled()) {
             w->suspendUnredirect(true);
@@ -296,81 +287,54 @@ void Scene::paintSimpleScreen(int orig_mask, QRegion region)
         }
         dirtyArea |= data.paint;
         // Schedule the window for painting
-        phase2data[w] = Phase2Data(w, data.paint, data.clip, data.mask, data.quads);
+        phase2data.append(QPair< Window*, Phase2Data >(w,Phase2Data(w, data.paint, data.clip,
+                                                                    data.mask, data.quads)));
         // no transformations, but translucency requires window pixmap
         w->suspendUnredirect(data.mask & PAINT_WINDOW_TRANSLUCENT);
     }
-    // Do the actual painting
-    // First opaque windows, top to bottom
-    // This also calculates correct paint regions for windows, also taking
-    //  care of clipping
+
+    // This is the occlusion culling pass
     QRegion allclips, upperTranslucentDamage;
-    for (int i = stacking_order.count() - 1; i >= 0; --i) {
-        QHash< Window*, Phase2Data >::iterator data = phase2data.find(stacking_order[ i ]);
-        if (data == phase2data.end())
-            continue;
+    for (int i = phase2data.count() - 1; i >= 0; --i) {
+        QPair< Window*, Phase2Data > *entry = &phase2data[i];
+        Phase2Data *data = &entry->second;
 
-        Toplevel *tlw = data.key()->window();
-        // In case there is a window with a higher stackposition which has translucent regions
-        // (e.g. decorations) that still have to be drawn, we also have to repaint the current window
-        // in these particular regions
-        data->region |= (upperTranslucentDamage & tlw->visibleRect());
+        data->region |= upperTranslucentDamage;
 
-        // subtract the parts which have possibly been drawn already as part of
+        // subtract the parts which will possibly been drawn as part of
         // a higher opaque window
         data->region -= allclips;
 
         // Here we rely on WindowPrePaintData::setTranslucent() to remove
         // the clip if needed.
-        if (!data->clip.isEmpty()) {
-            // clip away this region for all windows below this one
+        if (!data->clip.isEmpty() && !(data->mask & PAINT_WINDOW_TRANSFORMED)) {
+            // clip away the opaque regions for all windows below this one
             allclips |= data->clip;
-            // Paint the opaque window
-            data->painted_1stpass = data->clip & data->region;
-            paintWindow(data->window, data->mask, data->painted_1stpass, data->quads);
-            painted_region |= data->painted_1stpass;
-
-            // Clip out the client area, so we only draw the rest in the next pass
-            data->region -= data->clip;
-            // if prePaintWindow didn't change the clipping area we only have to paint
-            // the decoration
-            if ((data-> clip ^
-                 data.key()->clientShape().translated(data.key()->x(), data.key()->y())).isEmpty()) {
-                data->mask |= PAINT_DECORATION_ONLY;
-            }
+            // extend the translucent damage for windows below this by remaining (translucent) regions
+            upperTranslucentDamage |= data->region - data->clip;
+        } else {
+            upperTranslucentDamage |= data->region;
         }
-        // extend the translucent damage for windows below this by remaining (translucent) regions
-        upperTranslucentDamage |= data->region;
     }
 
-    QRegion painted_2ndpass;
+    QRegion paintedArea;
     // Fill any areas of the root window not covered by opaque windows
     if (!(orig_mask & PAINT_SCREEN_BACKGROUND_FIRST)) {
-        painted_2ndpass = dirtyArea - allclips;
-        paintBackground(painted_2ndpass);
+        paintedArea = dirtyArea - allclips;
+        paintBackground(paintedArea);
     }
 
-    // Now walk the list bottom to top, drawing translucent windows.
-    QRegion lowerOpaqueDamage;
-    for (int i = 0; i < stacking_order.count(); i++) {
-        QHash< Window*, Phase2Data >::iterator data = phase2data.find(stacking_order[ i ]);
-        if (data == phase2data.end())
-            continue;
+    // Now walk the list bottom to top and draw the windows.
+    for (int i = 0; i < phase2data.count(); ++i) {
+        Phase2Data *data = &phase2data[i].second;
 
-        Toplevel *tlw = data.key()->window();
-
-        // add all regions of the lower windows which have already be drawn in the 1st pass
-        data->region |= (lowerOpaqueDamage & tlw->visibleRect());
-        // and extend that region by opaque parts of ourself
-        lowerOpaqueDamage |= data->painted_1stpass;
-
-        // add all regions which have been drawn so far in the 2nd pass
-        painted_2ndpass |= data->region;
-        data->region = painted_2ndpass;
+        // add all regions which have been drawn so far
+        paintedArea |= data->region;
+        data->region = paintedArea;
 
         paintWindow(data->window, data->mask, data->region, data->quads);
     }
-    painted_region |= painted_2ndpass;
+    painted_region |= paintedArea;
 }
 
 void Scene::paintWindow(Window* w, int mask, QRegion region, WindowQuadList quads)
@@ -400,11 +364,46 @@ void Scene::paintWindow(Window* w, int mask, QRegion region, WindowQuadList quad
         thumbData.opacity = data.opacity;
 
         QSizeF size = QSizeF(thumb->size());
-        size.scale(QSizeF(item->sceneBoundingRect().width(), item->sceneBoundingRect().height()), Qt::KeepAspectRatio);
+        size.scale(QSizeF(item->width(), item->height()), Qt::KeepAspectRatio);
         thumbData.xScale = size.width() / static_cast<qreal>(thumb->width());
         thumbData.yScale = size.height() / static_cast<qreal>(thumb->height());
-        const int x = item->scenePos().x() + w->x() + (item->width() - size.width()) / 2;
-        const int y = item->scenePos().y() + w->y() + (item->height() - size.height()) / 2;
+        // it can happen in the init/closing phase of the tabbox
+        // that the corresponding QGraphicsScene is not available
+        if (item->scene() == 0) {
+            continue;
+        }
+        // in principle there could be more than one QGraphicsView per QGraphicsScene,
+        // although TabBox does not make use of it so far
+        QList<QGraphicsView*> views = item->scene()->views();
+        QGraphicsView* declview = 0;
+        QPoint viewPos;
+        foreach (QGraphicsView* view, views) {
+            if (view->winId() == w->window()->window()) {
+                declview = view;
+                break;
+            }
+            QWidget *parent = view;
+            while ((parent = parent->parentWidget())) {
+                // if the graphicsview is not the topmost widget we try to go up to the
+                // toplevel widget and check whether that is the window we are looking for.
+                if (parent->winId() == w->window()->window()) {
+                    declview = view;
+                    viewPos = view->mapTo(parent, QPoint());
+                    break;
+                }
+            }
+            if (declview) {
+                // our nested loop found it, so we can break this loop as well
+                // doesn't look nice, but still better than goto
+                break;
+            }
+        }
+        if (declview == 0) {
+            continue;
+        }
+        const QPoint point = viewPos + declview->mapFromScene(item->scenePos());
+        const qreal x = point.x() + w->x() + (item->width() - size.width())/2;
+        const qreal y = point.y() + w->y() + (item->height() - size.height()) / 2;
         thumbData.xTranslate = x - thumb->x();
         thumbData.yTranslate = y - thumb->y();
         int thumbMask = PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS;
@@ -413,13 +412,17 @@ void Scene::paintWindow(Window* w, int mask, QRegion region, WindowQuadList quad
         } else {
             thumbMask |= PAINT_WINDOW_TRANSLUCENT;
         }
-        if (x < wImpl->x() || x + size.width() > wImpl->x() + wImpl->width() ||
-            y < wImpl->y() || y + size.height() > wImpl->y() + wImpl->height()) {
-            // don't render windows outside the containing window.
-            // TODO: improve by spliting out the window quads which do not fit
-            continue;
+        QRegion clippingRegion = region;
+        clippingRegion &= QRegion(wImpl->x(), wImpl->y(), wImpl->width(), wImpl->height());
+        QPainterPath path = item->clipPath();
+        if (!path.isEmpty()) {
+            // here we assume that the clippath consists of a single rectangle
+            const QPolygonF sceneBounds = item->mapToScene(path.boundingRect());
+            const QRect viewBounds = declview->mapFromScene(sceneBounds).boundingRect();
+            // shrinking the rect due to rounding errors
+            clippingRegion &= viewBounds.adjusted(0,0,-1,-1).translated(viewPos + w->pos());
         }
-        effects->drawWindow(thumb, thumbMask, QRegion(x, y, size.width(), size.height()), thumbData);
+        effects->drawWindow(thumb, thumbMask, clippingRegion, thumbData);
     }
 }
 
@@ -443,6 +446,11 @@ OverlayWindow* Scene::overlayWindow()
     return m_overlayWindow;
 }
 
+void Scene::screenGeometryChanged(const QSize &size)
+{
+    m_overlayWindow->resize(size);
+}
+
 //****************************************
 // Scene::Window
 //****************************************
@@ -550,8 +558,8 @@ void Scene::Window::resetPaintingEnabled()
     if (Client* c = dynamic_cast< Client* >(toplevel)) {
         if (c->isMinimized())
             disable_painting |= PAINT_DISABLED_BY_MINIMIZE;
-        if (c->clientGroup() && c != c->clientGroup()->visible())
-            disable_painting |= PAINT_DISABLED_BY_CLIENT_GROUP;
+        if (c->tabGroup() && c != c->tabGroup()->current())
+            disable_painting |= PAINT_DISABLED_BY_TAB_GROUP;
         else if (c->isHiddenInternal())
             disable_painting |= PAINT_DISABLED;
     }
