@@ -51,7 +51,9 @@ public:
     virtual CompositingType compositingType() const = 0;
     // Repaints the given screen areas, windows provides the stacking order.
     // The entry point for the main part of the painting pass.
-    virtual void paint(QRegion damage, ToplevelList windows) = 0;
+    // returns the time since the last vblank signal - if there's one
+    // ie. "what of this frame is lost to painting"
+    virtual int paint(QRegion damage, ToplevelList windows) = 0;
 
     // Notification function - KWin core informs about changes.
     // Used to mainly discard cached data.
@@ -60,6 +62,13 @@ public:
     virtual void windowAdded(Toplevel*) = 0;
     // a window has been destroyed
     virtual void windowDeleted(Deleted*) = 0;
+    /**
+     * Method invoked when the screen geometry is changed.
+     * Reimplementing classes should also invoke the parent method
+     * as it takes care of resizing the overlay window.
+     * @param size The new screen geometry size
+     **/
+    virtual void screenGeometryChanged(const QSize &size);
     // Flags controlling how painting is done.
     enum {
         // Window (or at least part of it) will be painted opaque.
@@ -77,20 +86,15 @@ public:
         PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS = 1 << 5,
         // Clear whole background as the very first step, without optimizing it
         PAINT_SCREEN_BACKGROUND_FIRST = 1 << 6,
-        // Temporary solution since (_OPAQUE | _TRANSLUCENT) is not working currently.
-        PAINT_DECORATION_ONLY = 1 << 7,
+        // PAINT_DECORATION_ONLY = 1 << 7 has been removed
         // Window will be painted with a lanczos filter.
-        PAINT_WINDOW_LANCZOS = 1 << 8,
-        // same as PAINT_SCREEN_TRANSFORMED without full repainting
-        PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_WITHOUT_FULL_REPAINTS = 1 << 9
+        PAINT_WINDOW_LANCZOS = 1 << 8
+        // PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_WITHOUT_FULL_REPAINTS = 1 << 9 has been removed
     };
     // types of filtering available
     enum ImageFilterType { ImageFilterFast, ImageFilterGood };
-    inline uint estimatedRenderTime() {
-        return lastRenderTime;
-    }
     // there's nothing to paint (adjust time_diff later)
-    void idle();
+    virtual void idle();
     bool waitSyncAvailable() {
         return has_waitSync;
     }
@@ -134,7 +138,6 @@ protected:
         Window* window;
         QRegion region;
         QRegion clip;
-        QRegion painted_1stpass;
         int mask;
         WindowQuadList quads;
     };
@@ -148,7 +151,6 @@ protected:
     QRegion painted_region;
     // time since last repaint
     int time_diff;
-    uint lastRenderTime;
     QElapsedTimer last_time;
     Workspace* wspace;
     bool has_waitSync;
@@ -191,7 +193,7 @@ public:
         // Window will not be painted because it is minimized
         PAINT_DISABLED_BY_MINIMIZE     = 1 << 3,
         // Window will not be painted because it is not the active window in a client group
-        PAINT_DISABLED_BY_CLIENT_GROUP = 1 << 4,
+        PAINT_DISABLED_BY_TAB_GROUP = 1 << 4,
         // Window will not be painted because it's not on the current activity
         PAINT_DISABLED_BY_ACTIVITY     = 1 << 5
     };
