@@ -37,6 +37,7 @@ GLXDrawable SceneOpenGL::last_pixmap = None;
 
 SceneOpenGL::SceneOpenGL(Workspace* ws)
     : Scene(ws)
+    , m_resetModelViewProjectionMatrix(true)
     , init_ok(false)
 {
     initGLX();
@@ -54,9 +55,11 @@ SceneOpenGL::SceneOpenGL(Workspace* ws)
     if (!initRenderingContext())
         return; // error
     // Initialize OpenGL
+    GLPlatform *glPlatform = GLPlatform::instance();
+    glPlatform->detect();
+    glPlatform->printResults();
     initGL();
 
-    GLPlatform *glPlatform = GLPlatform::instance();
     if (glPlatform->isSoftwareEmulation()) {
         kError(1212) << "OpenGL Software Rasterizer detected. Falling back to XRender.";
         QTimer::singleShot(0, Workspace::self(), SLOT(fallbackToXRenderCompositing()));
@@ -75,15 +78,26 @@ SceneOpenGL::SceneOpenGL(Workspace* ws)
         glDrawBuffer(GL_BACK);
     // Check whether certain features are supported
     has_waitSync = false;
-    if (glXGetVideoSync && glXIsDirect(display(), ctxbuffer) && options->glVSync) {
-        unsigned int sync;
-        if (glXGetVideoSync(&sync) == 0) {
-            if (glXWaitVideoSync(1, 0, &sync) == 0)
-                has_waitSync = true;
-            else
-                qWarning() << "NO VSYNC! glXWaitVideoSync(1,0,&uint) isn't 0 but" << glXWaitVideoSync(1, 0, &sync);
+    if (options->isGlVSync()) {
+        if (glXGetVideoSync && glXSwapInterval && glXIsDirect(display(), ctxbuffer)) {
+            unsigned int sync;
+            if (glXGetVideoSync(&sync) == 0) {
+                if (glXWaitVideoSync(1, 0, &sync) == 0) {
+                    // NOTICE at this time we should actually check whether we can successfully
+                    // deactivate the swapInterval "glXSwapInterval(0) == 0"
+                    // (because we don't actually want it active unless we explicitly run a glXSwapBuffers)
+                    // However mesa/dri will return a range error (6) because deactivating the
+                    // swapinterval (as of today) seems completely unsupported
+                    has_waitSync = true;
+                    glXSwapInterval(0);
+                }
+                else
+                    qWarning() << "NO VSYNC! glXWaitVideoSync(1,0,&uint) isn't 0 but" << glXWaitVideoSync(1, 0, &sync);
+            } else
+                qWarning() << "NO VSYNC! glXGetVideoSync(&uint) isn't 0 but" << glXGetVideoSync(&sync);
         } else
-            qWarning() << "NO VSYNC! glXGetVideoSync(&uint) isn't 0 but" << glXGetVideoSync(&sync);
+            qWarning() << "NO VSYNC! glXGetVideoSync, glXSwapInterval, glXIsDirect" <<
+                        bool(glXGetVideoSync) << bool(glXSwapInterval) << glXIsDirect(display(), ctxbuffer);
     }
 
     debug = qstrcmp(qgetenv("KWIN_GL_DEBUG"), "1") == 0;
@@ -100,27 +114,16 @@ SceneOpenGL::SceneOpenGL(Workspace* ws)
     }
 
     // OpenGL scene setup
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    float fovy = 60.0f;
-    float aspect = 1.0f;
-    float zNear = 0.1f;
-    float zFar = 100.0f;
-    float ymax = zNear * tan(fovy  * M_PI / 360.0f);
-    float ymin = -ymax;
-    float xmin =  ymin * aspect;
-    float xmax = ymax * aspect;
-    // swap top and bottom to have OpenGL coordinate system match X system
-    glFrustum(xmin, xmax, ymin, ymax, zNear, zFar);
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
-    float scaleFactor = 1.1 * tan(fovy * M_PI / 360.0f) / ymax;
-    glTranslatef(xmin * scaleFactor, ymax * scaleFactor, -1.1);
-    glScalef((xmax - xmin)*scaleFactor / displayWidth(), -(ymax - ymin)*scaleFactor / displayHeight(), 0.001);
+    setupModelViewProjectionMatrix();
     if (checkGLError("Init")) {
         kError(1212) << "OpenGL compositing setup failed";
         return; // error
     }
+
+    // set strict binding
+    if (options->isGlStrictBindingFollowsDriver()) {
+        options->setGlStrictBinding(!glPlatform->supports(LooseBinding));
+    }
     kDebug(1212) << "DB:" << db << ", Direct:" << bool(glXIsDirect(display(), ctxbuffer)) << endl;
     init_ok = true;
 }
@@ -152,6 +155,28 @@ SceneOpenGL::~SceneOpenGL()
     checkGLError("Cleanup");
 }
 
+void SceneOpenGL::setupModelViewProjectionMatrix()
+{
+    glMatrixMode(GL_PROJECTION);
+    glLoadIdentity();
+    float fovy = 60.0f;
+    float aspect = 1.0f;
+    float zNear = 0.1f;
+    float zFar = 100.0f;
+    float ymax = zNear * tan(fovy  * M_PI / 360.0f);
+    float ymin = -ymax;
+    float xmin =  ymin * aspect;
+    float xmax = ymax * aspect;
+    // swap top and bottom to have OpenGL coordinate system match X system
+    glFrustum(xmin, xmax, ymin, ymax, zNear, zFar);
+    glMatrixMode(GL_MODELVIEW);
+    glLoadIdentity();
+    float scaleFactor = 1.1 * tan(fovy * M_PI / 360.0f) / ymax;
+    glTranslatef(xmin * scaleFactor, ymax * scaleFactor, -1.1);
+    glScalef((xmax - xmin)*scaleFactor / displayWidth(), -(ymax - ymin)*scaleFactor / displayHeight(), 0.001);
+    m_resetModelViewProjectionMatrix = false;
+}
+
 bool SceneOpenGL::initTfp()
 {
     if (glXBindTexImageEXT == NULL || glXReleaseTexImageEXT == NULL)
@@ -161,7 +186,7 @@ bool SceneOpenGL::initTfp()
 
 bool SceneOpenGL::initRenderingContext()
 {
-    bool direct_rendering = options->glDirect;
+    bool direct_rendering = options->isGlDirect();
     KXErrorHandler errs1;
     ctxbuffer = glXCreateNewContext(display(), fbcbuffer, GLX_RGBA_TYPE, NULL,
                                     direct_rendering ? GL_TRUE : GL_FALSE);
@@ -439,11 +464,12 @@ bool SceneOpenGL::initDrawableConfigs()
 }
 
 // the entry function for painting
-void SceneOpenGL::paint(QRegion damage, ToplevelList toplevels)
+int SceneOpenGL::paint(QRegion damage, ToplevelList toplevels)
 {
-    QElapsedTimer renderTimer;
-    renderTimer.start();
+    if (!m_lastDamage.isEmpty())
+        flushBuffer(m_lastMask, m_lastDamage);
 
+    // actually paint the frame, flushed with the NEXT frame
     foreach (Toplevel * c, toplevels) {
         assert(windows.contains(c));
         stacking_order.append(windows[ c ]);
@@ -451,38 +477,70 @@ void SceneOpenGL::paint(QRegion damage, ToplevelList toplevels)
 
     grabXServer();
     glXWaitX();
-    glPushMatrix();
+    if (m_resetModelViewProjectionMatrix) {
+        // reset model view projection matrix if required
+        setupModelViewProjectionMatrix();
+    }
     int mask = 0;
 #ifdef CHECK_GL_ERROR
     checkGLError("Paint1");
 #endif
     paintScreen(&mask, &damage);   // call generic implementation
+    m_lastMask = mask;
+    m_lastDamage = damage;
 #ifdef CHECK_GL_ERROR
     checkGLError("Paint2");
 #endif
-    glPopMatrix();
-    ungrabXServer(); // ungrab before flushBuffer(), it may wait for vsync
-    if (m_overlayWindow->window())  // show the window only after the first pass, since
-        m_overlayWindow->show();   // that pass may take long
-    lastRenderTime = renderTimer.elapsed();
-    flushBuffer(mask, damage);
+
+    glFlush();
+
+    ungrabXServer();
+    if (m_overlayWindow->window())  // show the window only after the first pass,
+        m_overlayWindow->show();   // since that pass may take long
+
     // do cleanup
     stacking_order.clear();
     checkGLError("PostPaint");
+    return m_renderTimer.elapsed();
 }
 
+#define VSYNC_DEBUG 0
+
 // wait for vblank signal before painting
 void SceneOpenGL::waitSync()
 {
     // NOTE that vsync has no effect with indirect rendering
     if (waitSyncAvailable()) {
+#if VSYNC_DEBUG
+        m_renderTimer.start();
+#endif
         uint sync;
-        glFlush();
+#if 0
+        // TODO: why precisely is this important?
+        // the sync counter /can/ perform multiple steps during glXGetVideoSync & glXWaitVideoSync
+        // but this only leads to waiting for two frames??!?
         glXGetVideoSync(&sync);
         glXWaitVideoSync(2, (sync + 1) % 2, &sync);
+#endif
+        glXWaitVideoSync(1, 0, &sync);
+#if VSYNC_DEBUG
+        static int waitTime = 0, waitCounter = 0, crapCounter = 0;
+        if (m_renderTimer.elapsed() > 11)
+            ++crapCounter;
+        waitTime += m_renderTimer.elapsed();
+        ++waitCounter;
+        if (waitCounter > 99)
+        {
+            qDebug() << "mean vsync wait time:" << float((float)waitTime / (float)waitCounter) << crapCounter << "/100";
+            crapCounter = waitTime = waitCounter = 0;
+        }
+#endif
     }
+    m_renderTimer.start(); // yes, the framerate shall be constant anyway.
 }
 
+#undef VSYNC_DEBUG
+
 // actually paint to the screen (double-buffer swap or copy from pixmap buffer)
 void SceneOpenGL::flushBuffer(int mask, QRegion damage)
 {
@@ -538,22 +596,53 @@ void SceneOpenGL::flushBuffer(int mask, QRegion damage)
                 }
             }
         } else {
-            waitSync();
-            glXSwapBuffers(display(), glxbuffer);
+            if (glXSwapInterval) {
+                glXSwapInterval(1);
+                glXSwapBuffers(display(), glxbuffer);
+                glXSwapInterval(0);
+                m_renderTimer.start(); // this is important so we don't assume to be loosing frames in the compositor timing calculation
+            } else {
+                waitSync();
+                glXSwapBuffers(display(), glxbuffer);
+            }
         }
         glXWaitGL();
-        XFlush(display());
     } else {
-        glFlush();
         glXWaitGL();
-        waitSync();
         if (mask & PAINT_SCREEN_REGION)
             foreach (const QRect & r, damage.rects())
-            XCopyArea(display(), buffer, rootWindow(), gcroot, r.x(), r.y(), r.width(), r.height(), r.x(), r.y());
+                XCopyArea(display(), buffer, rootWindow(), gcroot, r.x(), r.y(), r.width(), r.height(), r.x(), r.y());
         else
             XCopyArea(display(), buffer, rootWindow(), gcroot, 0, 0, displayWidth(), displayHeight(), 0, 0);
-        XFlush(display());
     }
+    XFlush(display());
+}
+
+void SceneOpenGL::screenGeometryChanged(const QSize &size)
+{
+    Scene::screenGeometryChanged(size);
+    glViewport(0,0, size.width(), size.height());
+    if (m_overlayWindow->window() == None) {
+        glXMakeCurrent(display(), None, NULL);
+        glXDestroyPixmap(display(), glxbuffer);
+        XFreePixmap(display(), buffer);
+        XVisualInfo* visual = glXGetVisualFromFBConfig(display(), fbcbuffer);
+        buffer = XCreatePixmap(display(), rootWindow(), size.width(), size.height(), visual->depth);
+        XFree(visual);
+        glxbuffer = glXCreatePixmap(display(), fbcbuffer, buffer, NULL);
+        glXMakeCurrent(display(), glxbuffer, ctxbuffer);
+        // TODO: there seems some bug, some clients become black until an eg. un/remap - could be a general pixmap buffer issue, though
+    }
+    else {
+        glXMakeCurrent(display(), None, NULL); // deactivate context ////
+        XMoveResizeWindow(display(), buffer, 0,0, size.width(), size.height());
+        m_overlayWindow->setup(buffer);
+        XSync(display(), false);  // ensure X11 stuff has applied ////
+        glXMakeCurrent(display(), glxbuffer, ctxbuffer); // reactivate context ////
+        glViewport(0,0, size.width(), size.height()); // adjust viewport last - should btw. be superflous on the Pixmap buffer - iirc glXCreatePixmap sets the context anyway. ////
+    }
+    ShaderManager::instance()->resetAllShaders();
+    m_resetModelViewProjectionMatrix = true;
 }
 
 //****************************************
@@ -566,14 +655,9 @@ SceneOpenGL::TexturePrivate::TexturePrivate()
 }
 
 SceneOpenGL::TexturePrivate::~TexturePrivate()
-{
-    release();
-}
-
-void SceneOpenGL::TexturePrivate::release()
 {
     if (m_glxpixmap != None) {
-        if (!options->glStrictBinding) {
+        if (!options->isGlStrictBinding()) {
             glXReleaseTexImageEXT(display(), m_glxpixmap, GLX_FRONT_LEFT_EXT);
         }
         glXDestroyPixmap(display(), m_glxpixmap);
@@ -617,6 +701,7 @@ void SceneOpenGL::Texture::findTarget()
 bool SceneOpenGL::Texture::load(const Pixmap& pix, const QSize& size,
                                 int depth, QRegion region)
 {
+    Q_UNUSED(region)
     // decrease the reference counter for the old texture
     d_ptr = new TexturePrivate();
 
@@ -680,34 +765,15 @@ bool SceneOpenGL::Texture::load(const Pixmap& pix, const QSize& size,
 #ifdef CHECK_GL_ERROR
     checkGLError("TextureLoad0");
 #endif
+    unbind();
     return true;
 }
 
-void SceneOpenGL::TexturePrivate::bind()
+void SceneOpenGL::TexturePrivate::onDamage()
 {
-    GLTexturePrivate::bind();
-    if (options->glStrictBinding && m_glxpixmap) {
+    if (options->isGlStrictBinding() && m_glxpixmap) {
         glXReleaseTexImageEXT(display(), m_glxpixmap, GLX_FRONT_LEFT_EXT);
         glXBindTexImageEXT(display(), m_glxpixmap, GLX_FRONT_LEFT_EXT, NULL);
-        m_hasValidMipmaps = false; // Mipmaps have to be regenerated after updating the texture
-    }
-    enableFilter();
-    if (hasGLVersion(1, 4, 0)) {
-        // Lod bias makes the trilinear-filtered texture look a bit sharper
-        glTexEnvf(GL_TEXTURE_FILTER_CONTROL, GL_TEXTURE_LOD_BIAS, -1.0f);
     }
+    GLTexturePrivate::onDamage();
 }
-
-void SceneOpenGL::TexturePrivate::unbind()
-{
-    if (hasGLVersion(1, 4, 0)) {
-        glTexEnvf(GL_TEXTURE_FILTER_CONTROL, GL_TEXTURE_LOD_BIAS, 0.0f);
-    }
-    if (options->glStrictBinding && m_glxpixmap) {
-        glBindTexture(m_target, m_texture);
-        glXReleaseTexImageEXT(display(), m_glxpixmap, GLX_FRONT_LEFT_EXT);
-    }
-
-    GLTexturePrivate::unbind();
-}
-
