@@ -21,27 +21,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // own
 #include "tabboxhandler.h"
 // tabbox
-#include "clientitemdelegate.h"
 #include "clientmodel.h"
 #include "declarative.h"
-#include "desktopitemdelegate.h"
 #include "desktopmodel.h"
-#include "itemlayoutconfig.h"
 #include "tabboxconfig.h"
-#include "tabboxview.h"
 // Qt
-#include <qdom.h>
-#include <QtCore/QtConcurrentRun>
-#include <QtCore/QFutureWatcher>
-#include <QFile>
 #include <QKeyEvent>
 #include <QModelIndex>
-#include <QPainter>
+#include <QTimer>
 #include <QX11Info>
 #include <X11/Xlib.h>
 // KDE
 #include <KDebug>
-#include <KStandardDirs>
 #include <KWindowSystem>
 
 namespace KWin
@@ -71,14 +62,12 @@ public:
 
     ClientModel* clientModel() const;
     DesktopModel* desktopModel() const;
-    void createView();
-    void parseConfig(const QString& fileName);
 
     TabBoxHandler *q; // public pointer
     // members
     TabBoxConfig config;
-    TabBoxView* view;
     DeclarativeView *m_declarativeView;
+    DeclarativeView *m_declarativeDesktopView;
     ClientModel* m_clientModel;
     DesktopModel* m_desktopModel;
     QModelIndex index;
@@ -87,13 +76,19 @@ public:
     * Used to determine if the outline has to be updated, etc.
     */
     bool isShown;
-    QMap< QString, ItemLayoutConfig > tabBoxLayouts;
     TabBoxClient *lastRaisedClient, *lastRaisedClientSucc;
+    WId m_embedded;
+    QPoint m_embeddedOffset;
+    QSize m_embeddedSize;
+    Qt::Alignment m_embeddedAlignment;
 };
 
 TabBoxHandlerPrivate::TabBoxHandlerPrivate(TabBoxHandler *q)
-    : view(NULL)
-    , m_declarativeView(NULL)
+    : m_declarativeView(NULL)
+    , m_declarativeDesktopView(NULL)
+    , m_embedded(0)
+    , m_embeddedOffset(QPoint(0, 0))
+    , m_embeddedSize(QSize(0, 0))
 {
     this->q = q;
     isShown = false;
@@ -102,38 +97,12 @@ TabBoxHandlerPrivate::TabBoxHandlerPrivate(TabBoxHandler *q)
     config = TabBoxConfig();
     m_clientModel = new ClientModel(q);
     m_desktopModel = new DesktopModel(q);
-
-    // load the layouts
-    QFuture< void> future = QtConcurrent::run(this, &TabBoxHandlerPrivate::parseConfig, KStandardDirs::locate("data", "kwin/DefaultTabBoxLayouts.xml"));
-    QFutureWatcher< void > *watcher = new QFutureWatcher< void >(q);
-    watcher->setFuture(future);
-    q->connect(watcher, SIGNAL(finished()), q, SIGNAL(ready()));
 }
 
 TabBoxHandlerPrivate::~TabBoxHandlerPrivate()
 {
-    delete view;
     delete m_declarativeView;
-}
-
-void TabBoxHandlerPrivate::createView()
-{
-    view = new TabBoxView(m_clientModel, m_desktopModel);
-    if (tabBoxLayouts.contains(config.layoutName())) {
-        view->clientDelegate()->setConfig(tabBoxLayouts.value(config.layoutName()));
-        view->desktopDelegate()->setConfig(tabBoxLayouts.value(config.layoutName()));
-    } else {
-        view->clientDelegate()->setConfig(tabBoxLayouts.value("Default"));
-        view->desktopDelegate()->setConfig(tabBoxLayouts.value("Desktop"));
-    }
-    if (tabBoxLayouts.contains(config.selectedItemLayoutName())) {
-        view->additionalClientDelegate()->setConfig(tabBoxLayouts.value(config.selectedItemLayoutName()));
-    } else {
-        view->additionalClientDelegate()->setConfig(tabBoxLayouts.value("Text"));
-    }
-    view->desktopDelegate()->setLayouts(tabBoxLayouts);
-    emit q->configChanged();
-    view->setCurrentIndex(index);
+    delete m_declarativeDesktopView;
 }
 
 ClientModel* TabBoxHandlerPrivate::clientModel() const
@@ -151,7 +120,7 @@ void TabBoxHandlerPrivate::updateOutline()
     if (config.tabBoxMode() != TabBoxConfig::ClientTabBox)
         return;
 //     if ( c == NULL || !m_isShown || !c->isShown( true ) || !c->isOnCurrentDesktop())
-    if (!isShown || clientModel()->data(index, ClientModel::EmptyRole).toBool()) {
+    if (!isShown) {
         q->hideOutline();
         return;
     }
@@ -166,8 +135,18 @@ void TabBoxHandlerPrivate::updateHighlightWindows()
 
     Display *dpy = QX11Info::display();
     TabBoxClient *currentClient = q->client(index);
+    QWidget *w = NULL;
+    if (m_declarativeView && m_declarativeView->isVisible()) {
+        w = m_declarativeView;
+    }
 
-    if (!KWindowSystem::compositingActive()) {
+    if (KWindowSystem::compositingActive()) {
+        if (lastRaisedClient)
+            q->elevateClient(lastRaisedClient, m_declarativeView ? m_declarativeView->winId() : 0, false);
+        lastRaisedClient = currentClient;
+        if (currentClient)
+            q->elevateClient(currentClient, m_declarativeView ? m_declarativeView->winId() : 0, true);
+    } else {
         if (lastRaisedClient) {
             if (lastRaisedClientSucc)
                 q->restack(lastRaisedClient, lastRaisedClientSucc);
@@ -193,12 +172,6 @@ void TabBoxHandlerPrivate::updateHighlightWindows()
 
     WId wId;
     QVector< WId > data;
-    QWidget *w = NULL;
-    if (view && view->isVisible()) {
-        w = view;
-    } else if (m_declarativeView && m_declarativeView->isVisible()) {
-        w = m_declarativeView;
-    }
     if (config.isShowTabBox() && w) {
         wId = w->winId();
         data.resize(2);
@@ -222,6 +195,9 @@ void TabBoxHandlerPrivate::updateHighlightWindows()
 
 void TabBoxHandlerPrivate::endHighlightWindows(bool abort)
 {
+    TabBoxClient *currentClient = q->client(index);
+    if (currentClient)
+        q->elevateClient(currentClient, m_declarativeView ? m_declarativeView->winId() : 0, false);
     if (abort && lastRaisedClient && lastRaisedClientSucc)
         q->restack(lastRaisedClient, lastRaisedClientSucc);
     lastRaisedClient = 0;
@@ -232,142 +208,6 @@ void TabBoxHandlerPrivate::endHighlightWindows(bool abort)
     XDeleteProperty(dpy, config.isShowTabBox() && m_declarativeView ? m_declarativeView->winId() : QX11Info::appRootWindow(), atom);
 }
 
-/***********************************************************
-* Based on the implementation of Kopete's
-* contaclistlayoutmanager.cpp by Nikolaj Hald Nielsen and
-* Roman Jarosz
-***********************************************************/
-void TabBoxHandlerPrivate::parseConfig(const QString& fileName)
-{
-    // open the file
-    if (!QFile::exists(fileName)) {
-        kDebug(1212) << "File " << fileName << " does not exist";
-        return;
-    }
-    QDomDocument doc("Layouts");
-    QFile file(fileName);
-    if (!file.open(QIODevice::ReadOnly)) {
-        kDebug(1212) << "Error reading file " << fileName;
-        return;
-    }
-    if (!doc.setContent(&file)) {
-        kDebug(1212) << "Error parsing file " << fileName;
-        file.close();
-        return;
-    }
-    file.close();
-
-    QDomElement layouts_element = doc.firstChildElement("tabbox_layouts");
-    QDomNodeList layouts = layouts_element.elementsByTagName("layout");
-
-    for (int i = 0; i < layouts.size(); i++) {
-        QDomNode layout = layouts.item(i);
-        ItemLayoutConfig currentLayout;
-
-        // parse top elements
-        QDomElement element = layout.toElement();
-        QString layoutName = element.attribute("name", "");
-
-        const bool highlightIcon = (element.attribute("highlight_selected_icon", "true").compare("true", Qt::CaseInsensitive) == 0);
-        currentLayout.setHighlightSelectedIcons(highlightIcon);
-
-        const bool grayscaleIcon = (element.attribute("grayscale_deselected_icon", "false").compare("true", Qt::CaseInsensitive) == 0);
-        currentLayout.setGrayscaleDeselectedIcons(grayscaleIcon);
-
-        // rows
-        QDomNodeList rows = element.elementsByTagName("row");
-        for (int j = 0; j < rows.size(); j++) {
-            QDomNode rowNode = rows.item(j);
-
-            ItemLayoutConfigRow row;
-
-            QDomNodeList elements = rowNode.toElement().elementsByTagName("element");
-            for (int k = 0; k < elements.size(); k++) {
-                QDomNode elementNode = elements.item(k);
-                QDomElement currentElement = elementNode.toElement();
-
-                ItemLayoutConfigRowElement::ElementType type = ItemLayoutConfigRowElement::ElementType(currentElement.attribute(
-                            "type", int(ItemLayoutConfigRowElement::ElementClientName)).toInt());
-                ItemLayoutConfigRowElement currentRowElement;
-                currentRowElement.setType(type);
-
-                // width - used by all types
-                qreal width = currentElement.attribute("width", "0.0").toDouble();
-                currentRowElement.setWidth(width);
-                switch(type) {
-                case ItemLayoutConfigRowElement::ElementEmpty:
-                    row.addElement(currentRowElement);
-                    break;
-                case ItemLayoutConfigRowElement::ElementIcon: {
-                    qreal iconWidth = currentElement.attribute("icon_size", "16.0").toDouble();
-                    currentRowElement.setIconSize(QSizeF(iconWidth, iconWidth));
-                    currentRowElement.setRowSpan(currentElement.attribute("row_span", "true").compare(
-                                                     "true", Qt::CaseInsensitive) == 0);
-                    row.addElement(currentRowElement);
-                    break;
-                }
-                case ItemLayoutConfigRowElement::ElementClientList: {
-                    currentRowElement.setStretch(currentElement.attribute("stretch", "false").compare(
-                                                     "true", Qt::CaseInsensitive) == 0);
-                    currentRowElement.setClientListLayoutName(currentElement.attribute("layout_name", ""));
-                    QString layoutMode = currentElement.attribute("layout_mode", "horizontal");
-                    if (layoutMode.compare("horizontal", Qt::CaseInsensitive) == 0)
-                        currentRowElement.setClientListLayoutMode(TabBoxConfig::HorizontalLayout);
-                    else if (layoutMode.compare("vertical", Qt::CaseInsensitive) == 0)
-                        currentRowElement.setClientListLayoutMode(TabBoxConfig::VerticalLayout);
-                    else if (layoutMode.compare("tabular", Qt::CaseInsensitive) == 0)
-                        currentRowElement.setClientListLayoutMode(TabBoxConfig::HorizontalVerticalLayout);
-                    row.addElement(currentRowElement);
-                    break;
-                }
-                default: { // text elements
-                    currentRowElement.setStretch(currentElement.attribute("stretch", "false").compare(
-                                                     "true", Qt::CaseInsensitive) == 0);
-                    currentRowElement.setSmallTextSize(currentElement.attribute("small", "false").compare(
-                                                           "true", Qt::CaseInsensitive) == 0);
-                    currentRowElement.setBold(currentElement.attribute("bold", "false").compare(
-                                                  "true", Qt::CaseInsensitive) == 0);
-                    currentRowElement.setItalic(currentElement.attribute("italic", "false").compare(
-                                                    "true", Qt::CaseInsensitive) == 0);
-                    currentRowElement.setItalicMinimized(currentElement.attribute("italic_minimized", "true").compare(
-                            "true", Qt::CaseInsensitive) == 0);
-
-                    currentRowElement.setPrefix(currentElement.attribute("prefix", ""));
-                    currentRowElement.setSuffix(currentElement.attribute("suffix", ""));
-                    currentRowElement.setPrefixMinimized(currentElement.attribute("prefix_minimized", ""));
-                    currentRowElement.setSuffixMinimzed(currentElement.attribute("suffix_minimized", ""));
-
-                    QString halign = currentElement.attribute("horizontal_alignment", "left");
-                    Qt::Alignment alignment;
-                    if (halign.compare("left", Qt::CaseInsensitive) == 0)
-                        alignment = Qt::AlignLeft;
-                    else if (halign.compare("right", Qt::CaseInsensitive) == 0)
-                        alignment = Qt::AlignRight;
-                    else
-                        alignment = Qt::AlignCenter;
-                    QString valign = currentElement.attribute("vertical_alignment", "center");
-                    if (valign.compare("top", Qt::CaseInsensitive) == 0)
-                        alignment = alignment | Qt::AlignTop;
-                    else if (valign.compare("bottom", Qt::CaseInsensitive) == 0)
-                        alignment = alignment | Qt::AlignBottom;
-                    else
-                        alignment = alignment | Qt::AlignVCenter;
-                    currentRowElement.setAlignment(alignment);
-
-                    row.addElement(currentRowElement);
-                    break;
-                }// case default
-                } // switch type
-            } // for loop elements
-
-            currentLayout.addRow(row);
-        } // for loop rows
-        if (!layoutName.isEmpty()) {
-            tabBoxLayouts.insert(layoutName, currentLayout);
-        }
-    } // for loop layouts
-}
-
 /***********************************************
 * TabBoxHandler
 ***********************************************/
@@ -391,18 +231,6 @@ const KWin::TabBox::TabBoxConfig& TabBoxHandler::config() const
 
 void TabBoxHandler::setConfig(const TabBoxConfig& config)
 {
-    if (config.layoutName() != d->config.layoutName()) {
-        // new item layout config
-        if (d->tabBoxLayouts.contains(config.layoutName()) && d->view) {
-            d->view->clientDelegate()->setConfig(d->tabBoxLayouts.value(config.layoutName()));
-            d->view->desktopDelegate()->setConfig(d->tabBoxLayouts.value(config.layoutName()));
-        }
-    }
-    if (config.selectedItemLayoutName() != d->config.selectedItemLayoutName()) {
-        // TODO: desktop layouts
-        if (d->tabBoxLayouts.contains(config.selectedItemLayoutName()) && d->view)
-            d->view->additionalClientDelegate()->setConfig(d->tabBoxLayouts.value(config.selectedItemLayoutName()));
-    }
     d->config = config;
     emit configChanged();
 }
@@ -420,23 +248,33 @@ void TabBoxHandler::show()
         if (d->config.tabBoxMode() == TabBoxConfig::ClientTabBox) {
             // use declarative view
             if (!d->m_declarativeView) {
-                d->m_declarativeView = new DeclarativeView(d->clientModel());
+                d->m_declarativeView = new DeclarativeView(d->clientModel(), TabBoxConfig::ClientTabBox);
             }
             d->m_declarativeView->show();
             d->m_declarativeView->setCurrentIndex(d->index, true);
         } else {
-            if (!d->view) {
-                d->createView();
+            if (!d->m_declarativeDesktopView) {
+                d->m_declarativeDesktopView = new DeclarativeView(d->desktopModel(), TabBoxConfig::DesktopTabBox);
             }
-            d->view->show();
-            d->view->updateGeometry();
+            d->m_declarativeDesktopView->show();
+            d->m_declarativeDesktopView->setCurrentIndex(d->index);
         }
     }
     if (d->config.isHighlightWindows()) {
-        d->updateHighlightWindows();
+        XSync(QX11Info::display(), false);
+        // TODO this should be
+        // QMetaObject::invokeMethod(this, "updateHighlightWindows", Qt::QueuedConnection);
+        // but we somehow need to cross > 1 event cycle (likely because of queued invocation in the effects)
+        // to ensure the EffectWindow is present when updateHighlightWindows, thus elevating the window/tabbox
+        QTimer::singleShot(1, this, SLOT(updateHighlightWindows()));
     }
 }
 
+void TabBoxHandler::updateHighlightWindows()
+{
+    d->updateHighlightWindows();
+}
+
 void TabBoxHandler::hide(bool abort)
 {
     d->isShown = false;
@@ -446,12 +284,12 @@ void TabBoxHandler::hide(bool abort)
     if (d->config.isShowOutline()) {
         hideOutline();
     }
-    if (d->view) {
-        d->view->hide();
-    }
     if (d->m_declarativeView) {
         d->m_declarativeView->hide();
     }
+    if (d->m_declarativeDesktopView) {
+        d->m_declarativeDesktopView->hide();
+    }
 }
 
 QModelIndex TabBoxHandler::nextPrev(bool forward) const
@@ -530,11 +368,6 @@ int TabBoxHandler::desktop(const QModelIndex& index) const
         return -1;
 }
 
-int TabBoxHandler::currentSelectedDesktop() const
-{
-    return desktop(d->index);
-}
-
 void TabBoxHandler::setCurrentIndex(const QModelIndex& index)
 {
     if (d->index == index) {
@@ -543,12 +376,12 @@ void TabBoxHandler::setCurrentIndex(const QModelIndex& index)
     if (!index.isValid()) {
         return;
     }
-    if (d->view) {
-        d->view->setCurrentIndex(index);
-    }
     if (d->m_declarativeView) {
         d->m_declarativeView->setCurrentIndex(index);
     }
+    if (d->m_declarativeDesktopView) {
+        d->m_declarativeDesktopView->setCurrentIndex(index);
+    }
     d->index = index;
     if (d->config.tabBoxMode() == TabBoxConfig::ClientTabBox) {
         if (d->config.isShowOutline()) {
@@ -558,6 +391,7 @@ void TabBoxHandler::setCurrentIndex(const QModelIndex& index)
             d->updateHighlightWindows();
         }
     }
+    emit selectedIndexChanged();
 }
 
 const QModelIndex& TabBoxHandler::currentIndex() const
@@ -565,80 +399,28 @@ const QModelIndex& TabBoxHandler::currentIndex() const
     return d->index;
 }
 
-QModelIndex TabBoxHandler::grabbedKeyEvent(QKeyEvent* event) const
+void TabBoxHandler::grabbedKeyEvent(QKeyEvent* event) const
 {
-    QModelIndex ret;
-    QAbstractItemModel* model;
-    switch(d->config.tabBoxMode()) {
-    case TabBoxConfig::ClientTabBox:
-        model = d->clientModel();
-        break;
-    case TabBoxConfig::DesktopTabBox:
-        model = d->desktopModel();
-        break;
-    default:
-        return d->index;
+    if (d->m_declarativeView && d->m_declarativeView->isVisible()) {
+        d->m_declarativeView->sendKeyEvent(event);
+    } else if (d->m_declarativeDesktopView && d->m_declarativeDesktopView->isVisible()) {
+        d->m_declarativeDesktopView->sendKeyEvent(event);
     }
-    int column = d->index.column();
-    int row = d->index.row();
-    switch(event->key()) {
-    case Qt::Key_Left:
-        column--;
-        if (column < 0)
-            column = model->columnCount() - 1;
-        break;
-    case Qt::Key_Right:
-        column++;
-        if (column >= model->columnCount())
-            column = 0;
-        break;
-    case Qt::Key_Up:
-        row--;
-        if (row < 0)
-            row = model->rowCount() - 1;
-        break;
-    case Qt::Key_Down:
-        row++;
-        if (row >= model->rowCount())
-            row = 0;
-        break;
-    default:
-        // do not do anything for any other key
-        break;
-    }
-    ret = model->index(row, column);
-
-    if (ret.isValid())
-        return ret;
-    else
-        return d->index;
 }
 
 bool TabBoxHandler::containsPos(const QPoint& pos) const
 {
     QWidget *w = NULL;
-    if (d->view && d->view->isVisible()) {
-        w = d->view;
-    } else if (d->m_declarativeView && d->m_declarativeView->isVisible()) {
+    if (d->m_declarativeView && d->m_declarativeView->isVisible()) {
         w = d->m_declarativeView;
+    } else if (d->m_declarativeDesktopView && d->m_declarativeDesktopView->isVisible()) {
+        w = d->m_declarativeDesktopView;
     } else {
         return false;
     }
     return w->geometry().contains(pos);
 }
 
-QModelIndex TabBoxHandler::indexAt(const QPoint& pos) const
-{
-    if (d->view && d->view->isVisible()) {
-        QPoint widgetPos = d->view->mapFromGlobal(pos);
-        return d->view->indexAt(widgetPos);
-    } else if (d->m_declarativeView && d->m_declarativeView->isVisible()) {
-        QPoint widgetPos = d->m_declarativeView->mapFromGlobal(pos);
-        return d->m_declarativeView->indexAt(widgetPos);
-    }
-    return QModelIndex();
-}
-
 QModelIndex TabBoxHandler::index(QWeakPointer<KWin::TabBox::TabBoxClient> client) const
 {
     return d->clientModel()->index(client);
@@ -654,8 +436,7 @@ TabBoxClientList TabBoxHandler::clientList() const
 TabBoxClient* TabBoxHandler::client(const QModelIndex& index) const
 {
     if ((!index.isValid()) ||
-            (d->config.tabBoxMode() != TabBoxConfig::ClientTabBox) ||
-            (d->clientModel()->data(index, ClientModel::EmptyRole).toBool()))
+            (d->config.tabBoxMode() != TabBoxConfig::ClientTabBox))
         return NULL;
     TabBoxClient* c = static_cast< TabBoxClient* >(
                           d->clientModel()->data(index, ClientModel::ClientRole).value<void *>());
@@ -692,9 +473,6 @@ void TabBoxHandler::createModel(bool partialReset)
         d->desktopModel()->createDesktopList();
         break;
     }
-    if (d->view) {
-        d->view->updateGeometry();
-    }
 }
 
 QModelIndex TabBoxHandler::first() const
@@ -713,9 +491,56 @@ QModelIndex TabBoxHandler::first() const
     return model->index(0, 0);
 }
 
-QWidget* TabBoxHandler::tabBoxView() const
+WId TabBoxHandler::embedded() const
 {
-    return d->view;
+    return d->m_embedded;
+}
+
+void TabBoxHandler::setEmbedded(WId wid)
+{
+    d->m_embedded = wid;
+    emit embeddedChanged(wid != 0);
+}
+
+void TabBoxHandler::setEmbeddedOffset(const QPoint &offset)
+{
+    d->m_embeddedOffset = offset;
+}
+
+void TabBoxHandler::setEmbeddedSize(const QSize &size)
+{
+    d->m_embeddedSize = size;
+}
+
+const QPoint &TabBoxHandler::embeddedOffset() const
+{
+    return d->m_embeddedOffset;
+}
+
+const QSize &TabBoxHandler::embeddedSize() const
+{
+    return d->m_embeddedSize;
+}
+
+Qt::Alignment TabBoxHandler::embeddedAlignment() const
+{
+    return d->m_embeddedAlignment;
+}
+
+void TabBoxHandler::setEmbeddedAlignment(Qt::Alignment alignment)
+{
+    d->m_embeddedAlignment = alignment;
+}
+
+void TabBoxHandler::resetEmbedded()
+{
+    if (d->m_embedded == 0) {
+        return;
+    }
+    d->m_embedded = 0;
+    d->m_embeddedOffset = QPoint(0, 0);
+    d->m_embeddedSize = QSize(0, 0);
+    emit embeddedChanged(false);
 }
 
 TabBoxHandler* tabBox = 0;
