@@ -32,10 +32,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QElapsedTimer>
 #include <kmanagerselection.h>
 
+// need to include utils.h before we use the ifdefs
+#include "utils.h"
+#ifdef KWIN_BUILD_ACTIVITIES
 #include <KActivities/Controller>
+#endif
 
 #include "plugins.h"
-#include "utils.h"
 #include "kdecoration.h"
 #include "kdecorationfactory.h"
 #ifdef KWIN_BUILD_SCREENEDGES
@@ -58,10 +61,6 @@ class KStartupInfoData;
 class QSlider;
 class QPushButton;
 
-namespace Kephal
-{
-    class Screen;
-}
 namespace KWin
 {
 
@@ -78,15 +77,12 @@ class Tile;
 class Tiling;
 class TilingLayout;
 #endif
-class ClientGroup;
-#ifdef KWIN_BUILD_DESKTOPCHANGEOSD
-class DesktopChangeOSD;
-#endif
 class Outline;
 class RootInfo;
 class PluginMgr;
 class Placement;
 class Rules;
+class Scripting;
 class WindowRules;
 
 class Workspace : public QObject, public KDecorationDefines
@@ -218,6 +214,10 @@ public:
      * @returns Total number of desktops currently in existence.
      */
     int numberOfDesktops() const;
+    /**
+     * @returns The maximum number of desktops that KWin supports.
+     */
+    int maxNumberOfDesktops() const;
     /**
      * Set the number of available desktops to @a count. This function overrides any previous
      * grid layout.
@@ -307,9 +307,10 @@ private:
     int* desktopGrid_;
     int currentDesktop_;
     QString activity_;
-    QStringList allActivities_;
-
+    QStringList allActivities_, openActivities_;
+#ifdef KWIN_BUILD_ACTIVITIES
     KActivities::Controller activityController_;
+#endif
 
 #ifdef KWIN_BUILD_TILING
     Tiling* m_tiling;
@@ -317,6 +318,7 @@ private:
     Outline* m_outline;
 #ifdef KWIN_BUILD_SCREENEDGES
     ScreenEdge m_screenEdge;
+    Qt::Orientations m_screenEdgeOrientation;
 #endif
 
     //-------------------------------------------------
@@ -335,10 +337,6 @@ public:
     QStringList activityList() const {
         return allActivities_;
     }
-    QStringList openActivityList() const {
-        return activityController_.listActivities(KActivities::Info::Running);
-    }
-
     // True when performing Workspace::updateClientArea().
     // The calls below are valid only in that case.
     bool inUpdateClientArea() const;
@@ -354,7 +352,7 @@ public:
     bool hasTabBox() const;
 
     const QVector<int> &desktopFocusChain() const {
-        return desktop_focus_chain;
+        return m_desktopFocusChain.value();
     }
     const ClientList &globalFocusChain() const {
         return global_focus_chain;
@@ -369,21 +367,11 @@ public:
         return client_keys;
     }
 
-    // Tabbing
-    void addClientGroup(ClientGroup* group);
-    void removeClientGroup(ClientGroup* group);
-    /// Returns the index of c in clientGroupList.
-    int indexOfClientGroup(ClientGroup* group);
-    /// Change the client c_id to the group with index g_id
-    void moveItemToClientGroup(ClientGroup* oldGroup, int oldIndex, ClientGroup* group, int index = -1);
-    Client* findSimilarClient(Client* c);
-    QList<ClientGroup*> clientGroups; // List of existing clients groups with no special order
-
     /**
      * Returns the list of clients sorted in stacking order, with topmost client
      * at the last position
      */
-    const ClientList& stackingOrder() const;
+    const ToplevelList& stackingOrder() const;
     ToplevelList xStackingOrder() const;
     ClientList ensureStackingOrder(const ClientList& clients) const;
 
@@ -420,6 +408,7 @@ public:
      */
     void showWindowMenu(int x, int y, Client* cl);
     void showWindowMenu(QPoint pos, Client* cl);
+    bool windowMenuShown();
 
     void updateMinimizedOfTransients(Client*);
     void updateOnAllDesktopsOfTransients(Client*);
@@ -442,7 +431,7 @@ public:
     bool hasDecorationShadows() const;
     Qt::Corner decorationCloseButtonCorner();
     bool decorationHasAlpha() const;
-    bool decorationSupportsClientGrouping() const; // Returns true if the decoration supports tabs.
+    bool decorationSupportsTabbing() const; // Returns true if the decoration supports tabs.
     bool decorationSupportsFrameOverlap() const;
     bool decorationSupportsBlurBehind() const;
 
@@ -462,6 +451,19 @@ public:
     bool stopActivity(const QString &id);
     bool startActivity(const QString &id);
     QStringList activeEffects() const;
+    QString supportInformation() const;
+    bool compositingPossible() const;
+    QString compositingNotPossibleReason() const;
+    bool openGLIsBroken() const;
+    /**
+     * Returns the currently used Compositor (Scene):
+     * @li @c none No Compositing
+     * @li @c xrender XRender
+     * @li @c gl1 OpenGL 1
+     * @li @c gl2 OpenGL 2
+     * @li @c gles OpenGL ES 2
+     **/
+    QString compositingType();
 
     void setCurrentScreen(int new_screen);
 
@@ -484,7 +486,7 @@ public:
 
     void removeUnmanaged(Unmanaged*, allowed_t);   // Only called from Unmanaged::release()
     void removeDeleted(Deleted*, allowed_t);
-    void addDeleted(Deleted*, allowed_t);
+    void addDeleted(Deleted*, Toplevel*, allowed_t);
 
     bool checkStartupNotification(Window w, KStartupInfoId& id, KStartupInfoData& data);
 
@@ -531,6 +533,10 @@ public:
     void addRepaint(int x, int y, int w, int h);
     void checkUnredirect(bool force = false);
     void checkCompositeTimer();
+    // returns the _estimated_ delay to the next screen update
+    // good for having a rough idea to calculate transformations, bad to rely on.
+    // might happen few ms earlier, might be an entire frame to short. This is NOT deterministic.
+    int nextFrameDelay();
 
     // Mouse polling
     void startMousePolling();
@@ -542,7 +548,6 @@ public:
 
 public slots:
     void addRepaintFull();
-    void refresh();
 
     // Keybindings
     void slotSwitchDesktopNext();
@@ -626,6 +631,7 @@ public slots:
     void slotSetupWindowShortcut();
     void setupWindowShortcutDone(bool);
     void slotToggleCompositing();
+    void slotInvertScreen();
 
     void updateClientArea();
     void suspendCompositing();
@@ -634,15 +640,15 @@ public slots:
     // NOTE: debug method
     void dumpTiles() const;
 
-    void slotSwitchToTabLeft(); // Slot to move left the active Client.
-    void slotSwitchToTabRight(); // Slot to move right the active Client.
-    void slotRemoveFromGroup(); // Slot to remove the active client from its group.
+    void slotActivateNextTab(); // Slot to move left the active Client.
+    void slotActivatePrevTab(); // Slot to move right the active Client.
+    void slotUntab(); // Slot to remove the active client from its group.
 
 private slots:
-    void groupTabPopupAboutToShow(); // Popup to add to another group
-    void switchToTabPopupAboutToShow(); // Popup to move in the group
-    void slotAddToTabGroup(QAction*);   // Add client to a group
-    void slotSwitchToTab(QAction*);   // Change the tab
+    void rebuildTabGroupPopup();
+    void rebuildTabListPopup();
+    void entabPopupClient(QAction*);
+    void selectPopupClientTab(QAction*);
     void desktopPopupAboutToShow();
     void activityPopupAboutToShow();
     void clientPopupAboutToShow();
@@ -652,10 +658,6 @@ private slots:
     void configureWM();
     void desktopResized();
     void screenChangeTimeout();
-    void screenAdded(Kephal::Screen*);
-    void screenRemoved(int);
-    void screenResized(Kephal::Screen*, QSize, QSize);
-    void screenMoved(Kephal::Screen*, QPoint, QPoint);
     void slotUpdateToolWindows();
     void delayFocus();
     void gotTemporaryRulesMessage(const QString&);
@@ -664,6 +666,10 @@ private slots:
     void slotBlockShortcuts(int data);
     void slotReloadConfig();
     void setupCompositing();
+    /**
+     * Called from setupCompositing() when the CompositingPrefs are ready.
+     **/
+    void slotCompositingOptionsInitialized();
     void finishCompositing();
     void fallbackToXRenderCompositing();
     void performCompositing();
@@ -671,26 +677,32 @@ private slots:
     void lostCMSelection();
     void resetCursorPosTime();
     void delayedCheckUnredirect();
-
     void updateCurrentActivity(const QString &new_activity);
     void activityRemoved(const QString &activity);
     void activityAdded(const QString &activity);
     void reallyStopActivity(const QString &id);   //dbus deadlocks suck
-
+    void handleActivityReply();
+    void showHideActivityMenu();
 protected:
     void timerEvent(QTimerEvent *te);
 
 Q_SIGNALS:
     Q_SCRIPTABLE void compositingToggled(bool active);
+    /**
+     * Emitted after the Workspace has setup the complete initialization process.
+     * This can be used to connect to for performing post-workspace initialization.
+     **/
+    void workspaceInitialized();
 
     //Signals required for the scripting interface
 signals:
     void desktopPresenceChanged(KWin::Client*, int);
-    void currentDesktopChanged(int);
+    void currentDesktopChanged(int, KWin::Client*);
     void numberDesktopsChanged(int oldNumberOfDesktops);
     void clientAdded(KWin::Client*);
     void clientRemoved(KWin::Client*);
     void clientActivated(KWin::Client*);
+    void clientDemandsAttentionChanged(KWin::Client*, bool);
     void groupAdded(KWin::Group*);
     void unmanagedAdded(KWin::Unmanaged*);
     void deletedRemoved(KWin::Deleted*);
@@ -705,11 +717,14 @@ private:
     void initShortcuts();
     void initDesktopPopup();
     void initActivityPopup();
+    void initTabbingPopups();
     void restartKWin(const QString &reason);
     void discardPopup();
     void setupWindowShortcut(Client* c);
     void checkCursorPos();
-
+#ifdef KWIN_BUILD_ACTIVITIES
+    void updateActivityList(bool running, bool updateCurrent, QString slot = QString());
+#endif
     enum Direction {
         DirectionNorth,
         DirectionEast,
@@ -719,7 +734,7 @@ private:
     void switchWindow(Direction direction);
 
     void propagateClients(bool propagate_new_clients);   // Called only from updateStackingOrder
-    ClientList constrainedStackingOrder();
+    ToplevelList constrainedStackingOrder();
     void raiseClientWithinApplication(Client* c);
     void lowerClientWithinApplication(Client* c);
     bool allowFullClientRaising(const Client* c, Time timestamp);
@@ -755,8 +770,11 @@ private:
 
     bool windowRepaintsPending() const;
     void setCompositeTimer();
+    int m_timeSinceLastVBlank, m_nextFrameDelay;
 
-    QVector<int> desktop_focus_chain;
+    typedef QHash< QString, QVector<int> > DesktopFocusChains;
+    DesktopFocusChains::Iterator m_desktopFocusChain;
+    DesktopFocusChains m_activitiesDesktopFocusChain;
 
     QWidget* active_popup;
     Client* active_popup_client;
@@ -795,8 +813,8 @@ private:
     UnmanagedList unmanaged;
     DeletedList deleted;
 
-    ClientList unconstrained_stacking_order; // Topmost last
-    ClientList stacking_order; // Topmost last
+    ToplevelList unconstrained_stacking_order; // Topmost last
+    ToplevelList stacking_order; // Topmost last
     bool force_restacking;
     mutable ToplevelList x_stacking; // From XQueryTree()
     mutable bool x_stacking_dirty;
@@ -821,9 +839,6 @@ private:
 #ifdef KWIN_BUILD_TABBOX
     TabBox::TabBox* tab_box;
 #endif
-#ifdef KWIN_BUILD_DESKTOPCHANGEOSD
-    DesktopChangeOSD* desktop_change_osd;
-#endif
 
     QMenu* popup;
     QMenu* advanced_popup;
@@ -848,16 +863,13 @@ private:
     QAction* mNoBorderOpAction;
     QAction* mMinimizeOpAction;
     QAction* mCloseOpAction;
-    QAction* mRemoveTabGroup; // Remove client from group
-    QAction* mCloseGroup; // Close all clients in the group
+    QAction* mRemoveFromTabGroup; // Remove client from group
+    QAction* mCloseTabGroup; // Close all clients in the group
     ShortcutDialog* client_keys_dialog;
     Client* client_keys_client;
     bool global_shortcuts_disabled;
     bool global_shortcuts_disabled_for_client;
 
-    void initAddToTabGroup(); // Load options for menu add_tabs_popup
-    void initSwitchToTab(); // Load options for menu switch_to_tab_popup
-
     PluginMgr* mgr;
 
     RootInfo* rootInfo;
@@ -902,9 +914,8 @@ private:
     KSelectionOwner* cm_selection;
     bool compositingSuspended, compositingBlocked;
     QBasicTimer compositeTimer;
-    QElapsedTimer nextPaintReference;
     QTimer mousePollingTimer;
-    uint vBlankInterval, vBlankPadding, fpsInterval, estimatedRenderTime;
+    uint vBlankInterval, fpsInterval;
     int xrrRefreshRate; // used only for compositing
     QRegion repaints_region;
     QSlider* transSlider;
@@ -914,6 +925,8 @@ private:
     QTimer compositeResetTimer; // for compressing composite resets
     bool m_finishingCompositing; // finishCompositing() sets this variable while shutting down
 
+    Scripting *m_scripting;
+
 private:
     friend bool performTransiencyCheck();
 };
@@ -1036,7 +1049,7 @@ inline void Workspace::removeGroup(Group* group, allowed_t)
     groups.removeAll(group);
 }
 
-inline const ClientList& Workspace::stackingOrder() const
+inline const ToplevelList& Workspace::stackingOrder() const
 {
     // TODO: Q_ASSERT( block_stacking_updates == 0 );
     return stacking_order;
@@ -1052,6 +1065,11 @@ inline void Workspace::showWindowMenu(int x, int y, Client* cl)
     showWindowMenu(QRect(QPoint(x, y), QPoint(x, y)), cl);
 }
 
+inline bool Workspace::windowMenuShown()
+{
+    return popup && ((QWidget*)popup)->isVisible();
+}
+
 inline void Workspace::setWasUserInteraction()
 {
     was_user_interaction = true;
@@ -1168,6 +1186,11 @@ inline void Workspace::checkCompositeTimer()
         setCompositeTimer();
 }
 
+inline int Workspace::nextFrameDelay()
+{
+    return m_nextFrameDelay;
+}
+
 inline bool Workspace::hasDecorationPlugin() const
 {
     if (!mgr) {
@@ -1200,12 +1223,12 @@ inline bool Workspace::decorationHasAlpha() const
     return mgr->factory()->supports(AbilityUsesAlphaChannel);
 }
 
-inline bool Workspace::decorationSupportsClientGrouping() const
+inline bool Workspace::decorationSupportsTabbing() const
 {
     if (!hasDecorationPlugin()) {
         return false;
     }
-    return mgr->factory()->supports(AbilityClientGrouping);
+    return mgr->factory()->supports(AbilityTabbing);
 }
 
 inline bool Workspace::decorationSupportsFrameOverlap() const
@@ -1224,11 +1247,6 @@ inline bool Workspace::decorationSupportsBlurBehind() const
     return mgr->factory()->supports(AbilityUsesBlurBehind);
 }
 
-inline void Workspace::addClientGroup(ClientGroup* group)
-{
-    clientGroups.append(group);
-}
-
 } // namespace
 
 #endif
